/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis 'TREMOR' CODEC SOURCE CODE.   *
 *                                                                  *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis 'TREMOR' SOURCE CODE IS (C) COPYRIGHT 1994-2002    *
 * BY THE Xiph.Org FOUNDATION http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: arm7 and later wide math functions

 ********************************************************************/

#define _ARM_ASSEM_ // already used.

#ifdef _ARM_ASSEM_

#if !defined(_V_WIDE_MATH) && !defined(_LOW_ACCURACY_)
#define _V_WIDE_MATH

for accuracy code blocked.

#endif

#ifndef _V_CLIP_MATH
#define _V_CLIP_MATH

static inline ogg_int32_t CLIP_TO_15(ogg_int32_t x) {
  int tmp;
  
  __asm {
    subs	tmp, x, #32768
    movpl	x, #0x7f00
    orrpl	x, x, #0xff
    adds	tmp, x, #32768
    movmi	x, #0x8000
  }
  
  return(x);
}

#endif

#ifndef _V_LSP_MATH_ASM
#define _V_LSP_MATH_ASM

static inline void lsp_loop_asm(ogg_uint32_t *qip,ogg_uint32_t *pip,
				ogg_int32_t *qexpp,
				ogg_int32_t *ilsp,ogg_int32_t wi,
				ogg_int32_t m){
  
  ogg_uint32_t qi=*qip,pi=*pip;
  ogg_int32_t qexp=*qexpp;
  
  int r0,r1,r2,r3;
  
  __asm {
    mov     r0,ilsp
    mov     r1,m,asr#1
    add     r0,r0,r1,lsl#3
    label1:
    
    ldmdb   r0!,{r1,r3}
    subs    r1,r1,wi          //ilsp[j]-wi
    rsbmi   r1,r1,#0          //labs(ilsp[j]-wi)
    umull   qi,r2,r1,qi       //qi*=labs(ilsp[j]-wi)
      
    subs    r1,r3,wi          //ilsp[j+1]-wi
    rsbmi   r1,r1,#0          //labs(ilsp[j+1]-wi)
    umull   pi,r3,r1,pi       //pi*=labs(ilsp[j+1]-wi)
      
    cmn     r2,r3             // shift down 16?
    beq     label0
    add     qexp,qexp,#16
    mov     qi,qi,lsr #16
    orr     qi,qi,r2,lsl #16
    mov     pi,pi,lsr #16
    orr     pi,pi,r3,lsl #16
    label0:
    cmp     r0,ilsp
    bhi     label1
      
      // odd filter assymetry
    ands    r0,m,#1
    beq     label2
    add     r0,ilsp,m,lsl#2
      
    ldr     r1,[r0,#-4]
    mov     r0,#0x4000
      
    subs    r1,r1,wi          //ilsp[j]-wi
    rsbmi   r1,r1,#0          //labs(ilsp[j]-wi)
    umull   qi,r2,r1,qi       //qi*=labs(ilsp[j]-wi)
    umull   pi,r3,r0,pi       //pi*=labs(ilsp[j+1]-wi)
      
    cmn     r2,r3             // shift down 16?
    beq     label2
    add     qexp,qexp,#16
    mov     qi,qi,lsr #16
    orr     qi,qi,r2,lsl #16
    mov     pi,pi,lsr #16
    orr     pi,pi,r3,lsl #16
      
      //qi=(pi>>shift)*labs(ilsp[j]-wi);
      //pi=(qi>>shift)*labs(ilsp[j+1]-wi);
      //qexp+=shift;
      
      //}
	 
      /* normalize to max 16 sig figs */
    label2:
    mov     r2,#0
    orr     r1,qi,pi
    tst     r1,#0xff000000
    addne   r2,r2,#8
    movne   r1,r1,lsr #8
    tst     r1,#0x00f00000
    addne   r2,r2,#4
    movne   r1,r1,lsr #4
    tst     r1,#0x000c0000
    addne   r2,r2,#2
    movne   r1,r1,lsr #2
    tst     r1,#0x00020000
    addne   r2,r2,#1
    movne   r1,r1,lsr #1
    tst     r1,#0x00010000
    addne   r2,r2,#1
    mov     qi,qi,lsr r2
    mov     pi,pi,lsr r2
    add     qexp,qexp,r2
  }
  
  *qip=qi;
  *pip=pi;
  *qexpp=qexp;
}

static inline void lsp_norm_asm(ogg_uint32_t *qip,ogg_int32_t *qexpp){

  ogg_uint32_t qi=*qip;
  ogg_int32_t qexp=*qexpp;

  __asm {
    tst     qi,#0x0000ff00
    moveq   qi,qi,lsl #8
    subeq   qexp,qexp,#8
    tst     qi,#0x0000f000
    moveq   qi,qi,lsl #4
    subeq   qexp,qexp,#4
    tst     qi,#0x0000c000
    moveq   qi,qi,lsl #2
    subeq   qexp,qexp,#2
    tst     qi,#0x00008000
    moveq   qi,qi,lsl #1
    subeq   qexp,qexp,#1
  }
  
  *qip=qi;
  *qexpp=qexp;
}

#endif
#endif

