
#include <NDS.h>

#include "../_const.h"
#include "../_console.h"

#include "../maindef.h"
#include "../memtool.h"
#include "../shell.h"
#include "../lang.h"
#include "../../../ipc3.h"

#include "global.h"
#include "helpscr.h"
#include "sound.h"

#include "component.h"

typedef struct {
  ECompIconType CompIconType;
  const char *Filename;
  CglB15 *pB15;
} TCompIconBM;

#define CompIconBMCount (ECITCount)
static TCompIconBM CompIconBMs[CompIconBMCount];

void Component_Init(void)
{
  for(u32 idx=0;idx<CompIconBMCount;idx++){
    TCompIconBM *pcibm=&CompIconBMs[idx];
    pcibm->CompIconType=(ECompIconType)idx;
    switch(pcibm->CompIconType){
      case ECIT_Ok: pcibm->Filename="comp_ok.b15"; break;
      case ECIT_Cancel: pcibm->Filename="comp_cancel.b15"; break;
      case ECIT_BoxMin: pcibm->Filename="comp_boxmin.b15"; break;
      case ECIT_BoxPlus: pcibm->Filename="comp_boxplus.b15"; break;
      case ECIT_CheckOff: pcibm->Filename="comp_chkoff.b15"; break;
      case ECIT_CheckOn: pcibm->Filename="comp_chkon.b15"; break;
      case ECIT_RadioOff: pcibm->Filename="comp_radiooff.b15"; break;
      case ECIT_RadioOn: pcibm->Filename="comp_radioon.b15"; break;
      case ECITCount: default: {
        _consolePrintf("Unknown EIconType value. %d\n",pcibm->CompIconType);
        ShowLogHalt();
      } break;
    }
    {
      u8 *pb15;
      int b15size;
      Shell_ReadCustomFile(pcibm->Filename,(void**)&pb15,&b15size);
      if(pb15!=NULL){
        pcibm->pB15=new CglB15(pb15,b15size);
        safefree(pb15); pb15=NULL;
      }
    }
  }
}

void Component_Free(void)
{
  for(u32 idx=0;idx<CompIconBMCount;idx++){
    TCompIconBM *pcibm=&CompIconBMs[idx];
    delete pcibm->pB15; pcibm->pB15=NULL;
  }
}

CglB15* Component_GetIcon(ECompIconType CompIconType)
{
  TCompIconBM *pcibm=&CompIconBMs[(u32)CompIconType];
  return(pcibm->pB15);
}

// -----------------------

void ComponentFrame_Init(TComponentFrame *pComponentFrame,CglCanvas *pCanvas)
{
  TComponentFrame *pcf=pComponentFrame;
  
  pcf->pCanvas=pCanvas;
  pcf->pTitleUTF8=NULL;
  pcf->Rect=CreateRect(0,0,0,0);
  pcf->FrameColor=(u16)(GlobalTextColor15<<1) | BIT15; // Kɂقǂ
  pcf->FillColor=RGB15(0,0,0)|BIT15;
}

void ComponentFrame_Draw(TComponentFrame *pComponentFrame)
{
  TComponentFrame *pcf=pComponentFrame;
  TRect r=pcf->Rect;
  
  if(21<pcf->Rect.y){
    const char *pTitleUTF8=pcf->pTitleUTF8;
    
    if(pTitleUTF8!=NULL){
      TRect r=pcf->Rect;
      r.h=20;
      r.y-=r.h;
      pcf->pCanvas->SetColor(GlobalTextColor15);
      pcf->pCanvas->DrawBox(r.x-1,r.y-1,r.w+2,r.h+1);
      pcf->pCanvas->SetColor(GlobalTitleBGColor15);
      pcf->pCanvas->FillBox(r.x,r.y,r.w,r.h);
      
      pcf->pCanvas->SetCglFont(pCglFont_Large14);
      pcf->pCanvas->SetFontBGColor(GlobalTitleBGColor15);
      pcf->pCanvas->SetFontTextColor(GlobalTextColor15);
      
      u32 TextHeight=pcf->pCanvas->GetTextHeight();
      pcf->pCanvas->TextOutUTF8(r.x+2,r.y+((r.h-TextHeight)/2),pTitleUTF8);
    }
  }
  
  pcf->pCanvas->SetColor(pcf->FrameColor);
  pcf->pCanvas->DrawBox(r.x-1,r.y-1,r.w+2,r.h+2);
  
  pcf->pCanvas->SetColor(pcf->FillColor|BIT15);
  pcf->pCanvas->FillBox(r.x,r.y,r.w,r.h);
}

// -----------------------

void ComponentLabel_Init(TComponentLabel *pComponentLabel,CglCanvas *pCanvas)
{
  TComponentLabel *pcl=pComponentLabel;
  
  pcl->CallBack_Click=NULL;
  pcl->pCanvas=pCanvas;
  pcl->Rect=CreateRect(0,0,0,0);
  pcl->pFont=pCglFont_Large14;
  pcl->BGColor=GlobalBGColor15;
  pcl->TextColor=GlobalTextColor15;
  pcl->Center=false;
  pcl->pMsgUTF8="";
  pcl->HelpFilename[0]=0;
}

void ComponentLabel_Draw(TComponentLabel *pComponentLabel)
{
  TComponentLabel *pcl=pComponentLabel;
  
  pcl->pCanvas->SetCglFont(pcl->pFont);
  pcl->pCanvas->SetFontBGColor(pcl->BGColor);
  pcl->pCanvas->SetFontTextColor(pcl->TextColor);
  
  const char *pmsgUTF8=pcl->pMsgUTF8;
  
  u32 TextWidth=pcl->pCanvas->GetTextWidthUTF8(pmsgUTF8);
  u32 TextHeight=pcl->pCanvas->GetTextHeight();
  
  if(pcl->Rect.w==0) pcl->Rect.w=TextWidth;
  if(pcl->Rect.h==0) pcl->Rect.h=TextHeight;
  
  TRect r=pcl->Rect;
  
  pcl->pCanvas->SetColor(pcl->BGColor);
  pcl->pCanvas->FillBox(r.x,r.y,r.w,r.h);
  
  if(pcl->Center==false){
    pcl->pCanvas->TextOutUTF8(r.x,r.y,pmsgUTF8);
    }else{
    pcl->pCanvas->TextOutUTF8(r.x+((r.w-TextWidth)/2),r.y,pmsgUTF8);
  }
}

s32 ComponentLabel_GetIndexFromPos(TComponentLabel *pComponentLabel,s32 mx,s32 my)
{
  TComponentLabel *pcl=pComponentLabel;
  TRect r=pcl->Rect;
  
  mx-=r.x;
  my-=r.y;
  
  if((0<=mx)&&(mx<r.w)){
    if((0<=my)&&(my<r.h)){
      return(0);
    }
  }
  
  return(-1);
}

bool ComponentLabel_MouseUp(TComponentLabel *pComponentLabel,s32 mx,s32 my)
{
  TComponentLabel *pcl=pComponentLabel;
  
  if(ComponentLabel_GetIndexFromPos(pcl,mx,my)==-1) return(false);
  
  if(pcl->CallBack_Click!=NULL) pcl->CallBack_Click(pcl);
  
  return(true);
}

// ------------------

void ComponentCheck_Init(TComponentCheck *pComponentCheck,CglCanvas *pCanvas)
{
  TComponentCheck *pcc=pComponentCheck;
  
  pcc->CallBack_Click=NULL;
  pcc->pCanvas=pCanvas;
  pcc->pOnCanvas=NULL;
  pcc->pOffCanvas=NULL;
  pcc->Checked=false;
  pcc->Rect=CreateRect(0,0,0,0);
  pcc->pFont=pCglFont_Large14;
  pcc->BGColor=GlobalBGColor15;
  pcc->TextColor=GlobalTextColor15;
  pcc->pMsgUTF8="";
  pcc->pData=NULL;
  pcc->DataValue=0;
  pcc->HelpFilename[0]=0;
}

void ComponentCheck_Draw(TComponentCheck *pComponentCheck)
{
  TComponentCheck *pcc=pComponentCheck;
  
  if(pcc->Rect.w==0) pcc->Rect.w=pcc->pOnCanvas->GetWidth();
  if(pcc->Rect.h==0) pcc->Rect.h=pcc->pOnCanvas->GetHeight();
  
  TRect r=pcc->Rect;
  
  CglCanvas *pSrcCanvas=NULL;
  if(pcc->Checked==true){
    pSrcCanvas=pcc->pOnCanvas;
    }else{
    pSrcCanvas=pcc->pOffCanvas;
  }
  if(pSrcCanvas!=NULL){
    s32 x=r.x;
    s32 y=r.y;
    s32 w=pSrcCanvas->GetWidth();
    s32 h=pSrcCanvas->GetHeight();
    x+=(r.w-w)/2;
    y+=(r.h-h)/2;
    pSrcCanvas->BitBlt(pcc->pCanvas,x,y,w,h,0,0,true);
  }
  
  pcc->pCanvas->SetCglFont(pcc->pFont);
  pcc->pCanvas->SetFontBGColor(pcc->BGColor);
  pcc->pCanvas->SetFontTextColor(pcc->TextColor);
  
  const char *pmsgUTF8=pcc->pMsgUTF8;
  
  u32 TextX=r.x+r.w+2;
  u32 TextY=r.y;
  u32 TextWidth=pcc->pCanvas->GetTextWidthUTF8(pmsgUTF8);
  u32 TextHeight=pcc->pCanvas->GetTextHeight();
  
  pcc->pCanvas->SetColor(pcc->BGColor);
  pcc->pCanvas->FillBox(TextX,TextY,TextWidth,TextHeight);
  
  pcc->pCanvas->TextOutUTF8(TextX,TextY,pmsgUTF8);
}

s32 ComponentCheck_GetIndexFromPos(TComponentCheck *pComponentCheck,s32 mx,s32 my)
{
  TComponentCheck *pcc=pComponentCheck;
  TRect r=pcc->Rect;
  
  pcc->pCanvas->SetCglFont(pcc->pFont);
  const char *pmsgUTF8=pcc->pMsgUTF8;
  s32 TextWidth=r.w+2+pcc->pCanvas->GetTextWidthUTF8(pmsgUTF8);
  
  mx-=r.x;
  my-=r.y;
  
  if((0<=mx)&&(mx<TextWidth)){
    if((0<=my)&&(my<r.h)){
      return(0);
    }
  }
  
  return(-1);
}

bool ComponentCheck_MouseUp(TComponentCheck *pComponentCheck,s32 mx,s32 my)
{
  TComponentCheck *pcc=pComponentCheck;
  
  if(ComponentCheck_GetIndexFromPos(pcc,mx,my)==-1) return(false);
  
  if(pcc->CallBack_Click!=NULL) pcc->CallBack_Click(pcc);
  
  return(true);
}

// ---------------------------------

// ------------------

void ComponentButton_Init(TComponentButton *pComponentButton,CglCanvas *pCanvas)
{
  TComponentButton *pcb=pComponentButton;
  
  pcb->CallBack_Click=NULL;
  pcb->pCanvas=pCanvas;
  pcb->pIconCanvas=NULL;
  pcb->Rect=CreateRect(0,0,0,0);
  pcb->pFont=pCglFont_Large14;
  pcb->BGColor=GlobalBGColor15;
  pcb->TextColor=GlobalTextColor15;
  pcb->pMsgUTF8="";
}

void ComponentButton_Draw(TComponentButton *pComponentButton)
{
  TComponentButton *pcb=pComponentButton;
  
  u16 BGColor;
  
  {
    BGColor=pcb->BGColor;
    TRect r=pcb->Rect;
    pcb->pCanvas->SetColor(RGB15(29,27,28)|BIT15);
    pcb->pCanvas->DrawBox(r.x-1,r.y-1,r.w+2,r.h+2);
    pcb->pCanvas->SetColor(RGB15(20,12,14)|BIT15);
    pcb->pCanvas->DrawBox(r.x,r.y,r.w+1,r.h+1);
    pcb->pCanvas->SetColor(BGColor);
    pcb->pCanvas->FillBox(r.x,r.y,r.w,r.h);
  }
  
  pcb->pCanvas->SetCglFont(pcb->pFont);
  pcb->pCanvas->SetFontBGColor(BGColor);
  pcb->pCanvas->SetFontTextColor(pcb->TextColor);
  
  const char *pmsgUTF8=pcb->pMsgUTF8;
  
  TRect r=pcb->Rect;
  
  s32 IconWidth;
  s32 IconHeight;
  if(pcb->pIconCanvas==NULL){
    IconWidth=0;
    IconHeight=0;
    }else{
    IconWidth=pcb->pIconCanvas->GetWidth();
    IconHeight=pcb->pIconCanvas->GetHeight();
  }
  
  s32 TextWidth=pcb->pCanvas->GetTextWidthUTF8(pmsgUTF8);
  s32 TextHeight=pcb->pCanvas->GetTextHeight();
  
  s32 BodyWidth=IconWidth+2+TextWidth;
  s32 BodyHeight=IconHeight;
  if(IconHeight<TextHeight) BodyHeight=TextHeight;
  s32 BodyX=r.x+((r.w-BodyWidth)/2);
  s32 BodyY=r.y+((r.h-BodyHeight)/2);
  
  if(pcb->pIconCanvas!=NULL){
    s32 x=BodyX;
    s32 y=BodyY+((BodyHeight-IconHeight)/2);
    pcb->pIconCanvas->BitBlt(pcb->pCanvas,x,y,IconWidth,IconHeight,0,0,true);
  }
  
  pcb->pCanvas->TextOutUTF8(BodyX+2+IconWidth,BodyY,pmsgUTF8);
}

s32 ComponentButton_GetIndexFromPos(TComponentButton *pComponentButton,s32 mx,s32 my)
{
  TComponentButton *pcb=pComponentButton;
  TRect r=pcb->Rect;
  
  mx-=r.x;
  my-=r.y;
  
  if((0<=mx)&&(mx<r.w)){
    if((0<=my)&&(my<r.h)){
      return(0);
    }
  }
  
  return(-1);
}

bool ComponentButton_MouseUp(TComponentButton *pComponentButton,s32 mx,s32 my)
{
  TComponentButton *pcb=pComponentButton;
  
  if(ComponentButton_GetIndexFromPos(pcb,mx,my)==-1) return(false);
  
  if(pcb->CallBack_Click!=NULL) pcb->CallBack_Click(pcb);
  
  return(true);
}

// ------------------ helper

char *pCompSectionName;
char *pCompItemName;

static void GetHelpFilename(char *pDstFilenameBuf,const char *pItemName)
{
  snprintf(pDstFilenameBuf,CustomFilenameMaxLength,"%s_%s.txt",pCompSectionName,pItemName);
}

static void ComponentLabelCallBack_Click(void *pComponentLabel)
{
  TComponentLabel *pcl=(TComponentLabel*)pComponentLabel;
  
  DrawHelpScreen_Item(pcl->pSectionName,pcl->pItemName,pcl->HelpFilename);
}

void SetCompNameLabel(TComponentLabel *pcl,u32 x,u32 y)
{
  pcl->CallBack_Click=ComponentLabelCallBack_Click;
  pcl->Rect=CreateRect(x,y,0,0);
  pcl->pMsgUTF8=Lang_GetItemNameFromINI(pCompSectionName,pCompItemName);
  if(pcl->pMsgUTF8==NULL) pcl->pMsgUTF8=pCompItemName;
  pcl->pSectionName=pCompSectionName;
  pcl->pItemName=pCompItemName;
  GetHelpFilename(pcl->HelpFilename,pCompItemName);
}

void SetCompValueLabel(TComponentLabel *pcl,u32 x,u32 y)
{
  pcl->CallBack_Click=ComponentLabelCallBack_Click;
  pcl->Rect=CreateRect(x,y,32,0);
  pcl->pMsgUTF8="";
  pcl->pSectionName=pCompSectionName;
  pcl->pItemName=pCompItemName;
  GetHelpFilename(pcl->HelpFilename,pCompItemName);
}

void SetCompUnitLabel(TComponentLabel *pcl,u32 x,u32 y)
{
  pcl->CallBack_Click=ComponentLabelCallBack_Click;
  pcl->Rect=CreateRect(x,y,0,0);
  pcl->pMsgUTF8=Lang_GetUnitNameFromINI(pCompSectionName,pCompItemName);
  if(pcl->pMsgUTF8==NULL) pcl->pMsgUTF8=pCompItemName;
  pcl->pSectionName=pCompSectionName;
  pcl->pItemName=pCompItemName;
  GetHelpFilename(pcl->HelpFilename,pCompItemName);
}

static void ComponentCheckCallBack_CheckClick(void *pComponentCheck)
{
  TComponentCheck *pcc=(TComponentCheck*)pComponentCheck;
  bool *pb=(bool*)pcc->pData;
  
  if(*pb==false){
    *pb=true;
    }else{
    *pb=false;
  }
  
  pcc->Checked=*pb;
  
  Sound_Start(ESID_Check);
  DrawHelpScreen_Item(pcc->pSectionName,pcc->pItemName,pcc->HelpFilename);
}

static void ComponentCheckCallBack_RadioClick(void *pComponentCheck)
{
  TComponentCheck *pcc=(TComponentCheck*)pComponentCheck;
  u8 *pu8=(u8*)pcc->pData;
  
  *pu8=pcc->DataValue;
  
  Sound_Start(ESID_Check);
  DrawHelpScreen_Item(pcc->pSectionName,pcc->pItemName,pcc->HelpFilename);
}

static void ComponentCheckCallBack_MinCheckClick(void *pComponentCheck)
{
  TComponentCheck *pcc=(TComponentCheck*)pComponentCheck;
  u32 *pu32=(u32*)pcc->pData;
  u32 v=pcc->DataValue;
  
  if(*pu32<v){
    *pu32=0;
    }else{
    *pu32-=v;
  }
  
  Sound_Start(ESID_Value);
  DrawHelpScreen_Item(pcc->pSectionName,pcc->pItemName,pcc->HelpFilename);
}

static void ComponentCheckCallBack_PlusCheckClick(void *pComponentCheck)
{
  TComponentCheck *pcc=(TComponentCheck*)pComponentCheck;
  u32 *pu32=(u32*)pcc->pData;
  u32 v=pcc->DataValue;
  
  *pu32+=v;
  
  Sound_Start(ESID_Value);
  DrawHelpScreen_Item(pcc->pSectionName,pcc->pItemName,pcc->HelpFilename);
}

void SetCompCheck(TComponentCheck *pcc,u32 x,u32 y,bool *pData)
{
  pcc->CallBack_Click=ComponentCheckCallBack_CheckClick;
  pcc->pOnCanvas=Component_GetIcon(ECIT_CheckOn)->pCanvas;
  pcc->pOffCanvas=Component_GetIcon(ECIT_CheckOff)->pCanvas;
  pcc->pMsgUTF8=Lang_GetItemNameFromINI(pCompSectionName,pCompItemName);
  if(pcc->pMsgUTF8==NULL) pcc->pMsgUTF8=pCompItemName;
  pcc->Rect=CreateRect(x,y,CompsCheckBoxSize,CompsCheckBoxSize);
  pcc->pData=pData;
  pcc->DataValue=0;
  pcc->pSectionName=pCompSectionName;
  pcc->pItemName=pCompItemName;
  GetHelpFilename(pcc->HelpFilename,pCompItemName);
  
  pcc->Checked=*pData;
}

void SetCompRadio(TComponentCheck *pcc,u32 x,u32 y,void *pData,s32 DataValue)
{
  pcc->CallBack_Click=ComponentCheckCallBack_RadioClick;
  pcc->pOnCanvas=Component_GetIcon(ECIT_RadioOn)->pCanvas;
  pcc->pOffCanvas=Component_GetIcon(ECIT_RadioOff)->pCanvas;
  pcc->pMsgUTF8=Lang_GetValueNameFromINI(pCompSectionName,pCompItemName,DataValue);
  if(pcc->pMsgUTF8==NULL){
    char *pmsg=(char*)safemalloc(128); // memory leak. for language makers.
    snprintf(pmsg,128,"%s%d",pCompItemName,DataValue);
    pcc->pMsgUTF8=pmsg;
  }
  pcc->Rect=CreateRect(x,y,CompsCheckBoxSize,CompsCheckBoxSize);
  pcc->pData=pData;
  pcc->DataValue=DataValue;
  pcc->pSectionName=pCompSectionName;
  pcc->pItemName=pCompItemName;
  GetHelpFilename(pcc->HelpFilename,pCompItemName);
}

void SetCompMinCheck(TComponentCheck *pcc,u32 x,u32 y,u32 *pData,s32 DataValue)
{
  pcc->CallBack_Click=ComponentCheckCallBack_MinCheckClick;
  pcc->pOnCanvas=Component_GetIcon(ECIT_BoxMin)->pCanvas;
  pcc->pOffCanvas=pcc->pOnCanvas;
  pcc->Checked=true;
  pcc->pMsgUTF8="";
  pcc->Rect=CreateRect(x,y,CompsCheckBoxSize,CompsCheckBoxSize);
  pcc->pData=pData;
  pcc->DataValue=DataValue;
  pcc->pSectionName=pCompSectionName;
  pcc->pItemName=pCompItemName;
  GetHelpFilename(pcc->HelpFilename,pCompItemName);
}

void SetCompPlusCheck(TComponentCheck *pcc,u32 x,u32 y,u32 *pData,s32 DataValue)
{
  pcc->CallBack_Click=ComponentCheckCallBack_PlusCheckClick;
  pcc->pOnCanvas=Component_GetIcon(ECIT_BoxPlus)->pCanvas;
  pcc->pOffCanvas=pcc->pOnCanvas;
  pcc->Checked=true;
  pcc->pMsgUTF8="";
  pcc->Rect=CreateRect(x,y,CompsCheckBoxSize,CompsCheckBoxSize);
  pcc->pData=pData;
  pcc->DataValue=DataValue;
  pcc->pSectionName=pCompSectionName;
  pcc->pItemName=pCompItemName;
  GetHelpFilename(pcc->HelpFilename,pCompItemName);
}

void SetCompTitleFrame(TComponentFrame *pcf)
{
  pcf->pTitleUTF8=Lang_GetItemNameFromINI(pCompSectionName,pCompItemName);
  if(pcf->pTitleUTF8==NULL) pcf->pTitleUTF8=pCompItemName;
}

void SetCompNameButton(TComponentButton *pcb)
{
  pcb->pMsgUTF8=Lang_GetItemNameFromINI(pCompSectionName,pCompItemName);
  if(pcb->pMsgUTF8==NULL) pcb->pMsgUTF8=pCompItemName;
}

