
#include <stdio.h>

#include <nds.h>

#include "../_console.h"
#include "../_consolewritelog.h"
#include "../maindef.h"
#include "../memtool.h"
#include "../_const.h"
#include "../inifile.h"
#include "../lang.h"
#include "../../../ipc3.h"

#include "../glib/glib.h"
#include "../shell.h"

#include "global.h"
#include "component.h"
#include "pagebtn.h"
#include "sound.h"

#include "custom.h"

CglFont *pCglFont_Large14;

void (*CallBack_Start)(void);
void (*CallBack_End)(void);
void (*CallBack_MouseDown)(s32 x,s32 y);
void (*CallBack_MouseMove)(s32 x,s32 y);
void (*CallBack_MouseUp)(s32 x,s32 y);

static void CallBackInit(void)
{
  CallBack_Start=NULL;
  CallBack_End=NULL;
  CallBack_MouseDown=NULL;
  CallBack_MouseMove=NULL;
  CallBack_MouseUp=NULL;
}

extern void ProcSystem1_SetCallBack(void);
extern void ProcSystem2_SetCallBack(void);
extern void ProcHiddenItem_SetCallBack(void);
extern void ProcKeyRepeat_SetCallBack(void);
extern void ProcBacklightTimeout_SetCallBack(void);
extern void ProcBoot_SetCallBack(void);
extern void ProcThumbnail_SetCallBack(void);
extern void ProcTextPlugin_SetCallBack(void);
extern void ProcImagePlugin1_SetCallBack(void);
extern void ProcImagePlugin2_SetCallBack(void);
extern void ProcGMEPlugin_SetCallBack(void);

extern void ProcExitDialog_SetCallBack(void);

void (*SetCallBacks[EMI_Count])(void)={
  ProcSystem1_SetCallBack,
  ProcSystem2_SetCallBack,
  ProcHiddenItem_SetCallBack,
  ProcKeyRepeat_SetCallBack,
  ProcBacklightTimeout_SetCallBack,
  ProcBoot_SetCallBack,
  ProcThumbnail_SetCallBack,
  ProcTextPlugin_SetCallBack,
  ProcImagePlugin1_SetCallBack,
  ProcImagePlugin2_SetCallBack,
  ProcGMEPlugin_SetCallBack,
  ProcExitDialog_SetCallBack,
};

static ESectionItem CurrentMenuPage;
ESectionItem NextMenuPage;

static u16 KEYS_CUR=0;
static s32 touchXpx,touchYpx;

void Proc_KeyInput(void)
{
}

enum emf {mf_none,mf_pagebar,mf_pagebody};

static emf mf=mf_none;
static s32 mx,my;

static void Proc_TouchPad(void)
{
//  _consolePrintf("%x,%d,%d\n",KEYS_CUR,IPC3->touchXpx,IPC3->touchYpx);
  
  if((KEYS_CUR&KEY_TOUCH)==0){
    if(mf!=mf_none){ cwl();
      switch(mf){
        case mf_none: break;
        case mf_pagebar: PageBtn_MouseUp(mx,my); break;
        case mf_pagebody: {
          if(CallBack_MouseUp!=NULL) CallBack_MouseUp(mx,my);
        } break;
      }
      mf=mf_none;
    }
    }else{ cwl();
    mx=touchXpx;
    my=touchYpx;
    
    if(mf==mf_none){ cwl();
      if(my<20){
        if(CurrentMenuPage!=EMI_Exit){
          mf=mf_pagebar;
          }else{
          mf=mf_pagebody;
        }
        }else{
        mf=mf_pagebody;
      }
      switch(mf){
        case mf_none: break;
        case mf_pagebar: PageBtn_MouseDown(mx,my); break;
        case mf_pagebody: {
          if(CallBack_MouseDown!=NULL) CallBack_MouseDown(mx,my);
        } break;
      }
      }else{ cwl();
      switch(mf){
        case mf_none: break;
        case mf_pagebar: PageBtn_MouseMove(mx,my); break;
        case mf_pagebody: {
          if(CallBack_MouseMove!=NULL) CallBack_MouseMove(mx,my);
        } break;
      }
    }
  }
}

void ScanIPC3(void)
{
  while(IPC3->ReqVsyncUpdate!=0){
    swiDelay(1);
  }
  
  KEYS_CUR=0;
  
  u32 btns=~IPC3->buttons;
  
  KEYS_CUR|=(~REG_KEYINPUT)&0x3ff;
  if((btns&IPC_PEN_DOWN)!=0) KEYS_CUR|=KEY_TOUCH;
  if((btns&IPC_X)!=0) KEYS_CUR|=KEY_X;
  if((btns&IPC_Y)!=0) KEYS_CUR|=KEY_Y;
  
  touchXpx=IPC3->touchXpx;
  touchYpx=IPC3->touchYpx;
  
  IPC3->ReqVsyncUpdate=1;
}

void custom_main(void)
{
  Sound_Init();
  Sound_Start(ESID_Open);
  
  Shell_SetCustomLangID(IPC3->UserLanguage);
  
  {
    char *pbuf;
    s32 size;
    Shell_ReadCustomLangFile("_lang.ini",(void**)&pbuf,&size);
    if(pbuf!=NULL){
      Lang_Init(pbuf,size);
      safefree(pbuf); pbuf=NULL;
    }
  }
  
  {
    u8 *pglf;
    int glfsize;
    Shell_ReadCustomLangFile("_font.glf",(void**)&pglf,&glfsize);
    if(pglf==NULL){
      _consolePrint("Can not found font file.\nWas the resource for the custom installed?\n");
      ShowLogHalt();
    }
    pCglFont_Large14=new CglFont(pglf,glfsize);
  }
  
  {
    TiniCustom *Custom=&GlobalINI.Custom;
    if(EMI_Count<=Custom->PageIndex) Custom->PageIndex=0;
    
    PageBtn_Init(Custom->PageIndex,Custom->PageOffset);
    Component_Init();
    
    CurrentMenuPage=(ESectionItem)Custom->PageIndex;
    NextMenuPage=(ESectionItem)-1;
  }
  
  while(1){
    PageBtn_SetSelectIndex(CurrentMenuPage);
    PageBtn_TransScreen();
    
    CallBackInit();
    SetCallBacks[CurrentMenuPage]();
    
    if(CallBack_Start!=NULL) CallBack_Start();
    
    while(NextMenuPage==(ESectionItem)-1){
      ScanIPC3();
      //swiWaitForVBlank();
      Proc_TouchPad();
      Proc_KeyInput();
    }
    
    if(CallBack_End!=NULL) CallBack_End();
    
    if(CurrentMenuPage!=EMI_Exit){
      TiniCustom *Custom=&GlobalINI.Custom;
      Custom->PageIndex=PageBtn_GetPageIndex();
      Custom->PageOffset=PageBtn_GetPageOffset();
    }
    
    if(NextMenuPage==EMI_Exit){
      Sound_Start(ESID_Dialog);
      }else{
      if(NextMenuPage!=EMI_Exit) Sound_Start(ESID_Page);
    }
    
    CurrentMenuPage=(ESectionItem)NextMenuPage;
    NextMenuPage=(ESectionItem)-1;
  }
  
  Component_Free();
  Sound_Init();
  
  while(1);
}

