
#include <stdio.h>
#include <nds.h>

#include "../_const.h"
#include "../_console.h"

#include "../maindef.h"
#include "../memtool.h"
#include "../shell.h"
#include "../../../ipc3.h"

#include "global.h"

#include "helpscr.h"

u32 hex2int(char hex)
{
  if(('0'<=hex)&&(hex<='9')) return((u32)(hex-'0'));
  if(('a'<=hex)&&(hex<='f')) return(0x10+(u32)(hex-'a'));
  if(('A'<=hex)&&(hex<='F')) return(0x10+(u32)(hex-'A'));
  return(0);
}

static void DrawTextFile(CglCanvas *pcan,u32 x,u32 y,const char *pFilename)
{
  if(pFilename==NULL) return;
  
  char *pbuf=NULL;
  s32 size=0;
  s32 ofs=0;
  
  Shell_ReadCustomLangFile(pFilename,(void**)&pbuf,&size);
  if(pbuf==NULL) return;
  
  char linebuf[256];
  u32 linelen=0;
  linebuf[linelen]=0;
  
  while(ofs<size){
    if(pbuf[ofs]==0) break;
    
    if(pbuf[ofs]<0x20){
      if(pbuf[ofs]==0x0d){
        linebuf[linelen]=0;
        
        if(linelen!=0){
          u32 ofs=0;
          u16 col=GlobalTextColor15;
          if(linebuf[0]=='#'){
            u32 r=(hex2int(linebuf[1])<<4) | hex2int(linebuf[2]);
            u32 g=(hex2int(linebuf[3])<<4) | hex2int(linebuf[4]);
            u32 b=(hex2int(linebuf[5])<<4) | hex2int(linebuf[6]);
            col=RGB15(r>>3,g>>3,b>>3) | BIT15;
            ofs=7;
          }
          pcan->SetFontTextColor(col);
          pcan->TextOutUTF8(x,y,&linebuf[ofs]);
        }
        
        linelen=0;
        
        y+=16;
        if((ScreenHeight-(u32)pcan->GetTextHeight())<y) break;
      }
      }else{
      linebuf[linelen++]=pbuf[ofs];
    }
    ofs++;
  }
  
  safefree(pbuf); pbuf=NULL;
}

void DrawHelpScreen_Section(const char *pSectionName)
{
  CglCanvas *pcan=new CglCanvas(NULL,ScreenWidth,ScreenHeight,pf15bit);
  
  pcan->SetColor(GlobalBGColor15);
  pcan->FillBox(0,0,ScreenWidth,ScreenHeight);
  
  pcan->SetCglFont(pCglFont_Large14);
  pcan->SetFontBGColor(GlobalBGColor15);
  pcan->SetFontTextColor(GlobalTextColor15);
  
  u32 x=8,y=12;
  
  if(pSectionName!=NULL){
    char s[256];
    snprintf(s,256,"[%s] section",pSectionName);
    pcan->TextOutUTF8(x,y,s);
    y+=16;
  }
  
  y+=20;
  
  char fn[CustomFilenameMaxLength];
  snprintf(fn,CustomFilenameMaxLength,"%s.txt",pSectionName);
  DrawTextFile(pcan,x,y,fn);
  
  pcan->BitBlt(pScreenSub->pCanvas,0,0,ScreenWidth,ScreenHeight,0,0,false);
  
  delete pcan; pcan=NULL;
}

void DrawHelpScreen_Item(const char *pSectionName,const char *pItemName,const char *pHelpFilename)
{
  CglCanvas *pcan=new CglCanvas(NULL,ScreenWidth,ScreenHeight,pf15bit);
  
  swiWaitForVBlank();
  pcan->SetColor(GlobalBGColor15);
  pcan->FillBox(0,0,ScreenWidth,ScreenHeight);
  swiWaitForVBlank();
  
  pcan->SetCglFont(pCglFont_Large14);
  pcan->SetFontBGColor(GlobalBGColor15);
  pcan->SetFontTextColor(GlobalTextColor15);
  
  
  u32 x=8,y=12;
  
  if(pSectionName!=NULL){
    char s[256];
    snprintf(s,256,"[%s] section",pSectionName);
    pcan->TextOutUTF8(x,y,s);
    y+=16;
  }
  
  if(pItemName!=NULL){
    pcan->TextOutUTF8(x,y,pItemName);
    y+=16;
  }
  
  y+=20;
  DrawTextFile(pcan,x,y,pHelpFilename);
  
  pcan->BitBlt(pScreenSub->pCanvas,0,0,ScreenWidth,ScreenHeight,0,0,false);
  
  delete pcan; pcan=NULL;
}

