
#include <nds.h>

#include "../_const.h"
#include "../_console.h"
#include "../inifile.h"
#include "../memtool.h"
#include "../lang.h"

#include "global.h"
#include "rect.h"
#include "component.h"
#include "helpscr.h"

enum ECompLabels {ECLS_StartupSound,ECLSCount};
#define CompLabelsCount (ECLSCount)
static TComponentLabel *pCompLabels;

enum ECompChecks {ECCS_hiddenAboutWindow,ECCS_hiddenHelpWindow,ECCS_hiddenDateTimeWindow,
                  ECCS_StartupSound0,ECCS_StartupSound1,ECCS_StartupSound2,
                  ECCSCount};
#define CompChecksCount (ECCSCount)
static TComponentCheck *pCompChecks;

static void CompsReload(void)
{
  TiniBoot *pi=&GlobalINI.Boot;
  
  pCompChecks[ECCS_StartupSound0].Checked=false;
  pCompChecks[ECCS_StartupSound1].Checked=false;
  pCompChecks[ECCS_StartupSound2].Checked=false;
  switch(pi->StartupSound){
    case 0: pCompChecks[ECCS_StartupSound0].Checked=true; break;
    case 1: pCompChecks[ECCS_StartupSound1].Checked=true; break;
    case 2: pCompChecks[ECCS_StartupSound2].Checked=true; break;
  }
}

static void CompsRedraw(bool FullDraw)
{
  if(FullDraw==true){
    CglCanvas *pcan=pScreenMainOverlay->pCanvas;
    
    u32 MenuBarHeight=20;
    pcan->SetColor(GlobalBGColor15);
    pcan->FillBox(0,MenuBarHeight,ScreenWidth,ScreenHeight-MenuBarHeight);
  }
  
  if(FullDraw==true){
    for(u32 idx=0;idx<CompLabelsCount;idx++){
      ComponentLabel_Draw(&pCompLabels[idx]);
    }
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
    }else{
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
  }
}

static void CompsInit(void)
{
  pCompLabels=(TComponentLabel*)safemalloc(sizeof(TComponentLabel)*CompLabelsCount);
  pCompChecks=(TComponentCheck*)safemalloc(sizeof(TComponentCheck)*CompChecksCount);
  
  CglCanvas *pcan=pScreenMainOverlay->pCanvas;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_Init(&pCompLabels[idx],pcan);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    ComponentCheck_Init(&pCompChecks[idx],pcan);
  }
  
  TiniBoot *pi=&GlobalINI.Boot;
  pCompSectionName="Boot";
  
  s32 x,y;
  
  x=8; y=20;
  
  {
    pCompItemName="hiddenAboutWindow";
    y+=CompsLinePadding;
    SetCompCheck(&pCompChecks[ECCS_hiddenAboutWindow],x,y,&pi->hiddenAboutWindow);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="hiddenHelpWindow";
    y+=CompsLinePadding;
    SetCompCheck(&pCompChecks[ECCS_hiddenHelpWindow],x,y,&pi->hiddenHelpWindow);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="hiddenDateTimeWindow";
    y+=CompsLinePadding;
    SetCompCheck(&pCompChecks[ECCS_hiddenDateTimeWindow],x,y,&pi->hiddenDateTimeWindow);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="StartupSound";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_StartupSound],x,y);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_StartupSound0],x,y,&pi->StartupSound,0);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_StartupSound1],x,y,&pi->StartupSound,1);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_StartupSound2],x,y,&pi->StartupSound,2);
  }
  
}

static void CompsFree(void)
{
  if(pCompLabels!=NULL){
    safefree(pCompLabels); pCompLabels=NULL;
  }
  if(pCompChecks!=NULL){
    safefree(pCompChecks); pCompChecks=NULL;
  }
}

// -----------------------------

static bool deskmf;

static void CB_MouseDown(s32 x,s32 y)
{
  deskmf=true;
}

static void CB_MouseMove(s32 x,s32 y)
{
}

static void CB_MouseUp(s32 x,s32 y)
{
  if(deskmf==false) return;
  deskmf=false;
  
  bool redraw=false;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_MouseUp(&pCompLabels[idx],x,y);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    if(ComponentCheck_MouseUp(&pCompChecks[idx],x,y)==true) redraw=true;
  }
  
  if(redraw==true){
    CompsReload();
    CompsRedraw(false);
  }
}

static void CB_Start(void)
{
  deskmf=false;
  
  DrawHelpScreen_Section("Boot");
  
  CompsInit();
  CompsReload();
  CompsRedraw(true);
}

static void CB_End(void)
{
  CompsFree();
}

void ProcBoot_SetCallBack(void)
{
  CallBack_Start=CB_Start;
  CallBack_End=CB_End;
  CallBack_MouseDown=CB_MouseDown;
  CallBack_MouseMove=CB_MouseMove;
  CallBack_MouseUp=CB_MouseUp;
}


