
#include <nds.h>

#include "../_const.h"
#include "../_console.h"
#include "../inifile.h"
#include "../memtool.h"
#include "../lang.h"

#include "global.h"
#include "rect.h"
#include "component.h"
#include "helpscr.h"

enum ECompLabels {ECLS_VerticalPadding,ECLS_VerticalPaddingUnit,ECLS_VerticalPaddingVal,
                  ECLS_Interpolation,
                  ECLSCount};
#define CompLabelsCount (ECLSCount)
static TComponentLabel *pCompLabels;

enum ECompChecks {ECCS_VerticalPaddingMin,ECCS_VerticalPaddingPlus,
                  ECCS_Interpolation0,ECCS_Interpolation1,
                  ECCSCount};
#define CompChecksCount (ECCSCount)
static TComponentCheck *pCompChecks;

static void CompsReload(void)
{
  TiniImagePlugin *pi=&GlobalINI.ImagePlugin;
  
  {
    static char val[8];
    snprintf(val,8,"%d",pi->VerticalPadding);
    pCompLabels[ECLS_VerticalPaddingVal].pMsgUTF8=val;
  }
  
  pCompChecks[ECCS_Interpolation0].Checked=false;
  pCompChecks[ECCS_Interpolation1].Checked=false;
  switch(pi->Interpolation){
    case 0: pCompChecks[ECCS_Interpolation0].Checked=true; break;
    case 1: pCompChecks[ECCS_Interpolation1].Checked=true; break;
  }
  
}

static void CompsRedraw(bool FullDraw)
{
  if(FullDraw==true){
    CglCanvas *pcan=pScreenMainOverlay->pCanvas;
    
    u32 MenuBarHeight=20;
    pcan->SetColor(GlobalBGColor15);
    pcan->FillBox(0,MenuBarHeight,ScreenWidth,ScreenHeight-MenuBarHeight);
  }
  
  if(FullDraw==true){
    for(u32 idx=0;idx<CompLabelsCount;idx++){
      ComponentLabel_Draw(&pCompLabels[idx]);
    }
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
    }else{
    ComponentLabel_Draw(&pCompLabels[ECLS_VerticalPaddingVal]);
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
  }
}

static void CompsInit(void)
{
  pCompLabels=(TComponentLabel*)safemalloc(sizeof(TComponentLabel)*CompLabelsCount);
  pCompChecks=(TComponentCheck*)safemalloc(sizeof(TComponentCheck)*CompChecksCount);
  
  CglCanvas *pcan=pScreenMainOverlay->pCanvas;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_Init(&pCompLabels[idx],pcan);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    ComponentCheck_Init(&pCompChecks[idx],pcan);
  }
  
  TiniImagePlugin *pi=&GlobalINI.ImagePlugin;
  pCompSectionName="ImagePlugin";
  
  s32 x,y;
  
  x=8; y=20;
  
  {
    pCompItemName="VerticalPadding";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_VerticalPadding],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_VerticalPaddingMin],x+128,y,&pi->VerticalPadding,1);
    SetCompValueLabel(&pCompLabels[ECLS_VerticalPaddingVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_VerticalPaddingPlus],x+128+CompsCheckBoxSize+32,y,&pi->VerticalPadding,1);
    SetCompUnitLabel(&pCompLabels[ECLS_VerticalPaddingUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="Interpolation";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_Interpolation],x,y);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Interpolation0],x,y,&pi->Interpolation,0);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Interpolation1],x,y,&pi->Interpolation,1);
  }
  
}

static void CompsFree(void)
{
  if(pCompLabels!=NULL){
    safefree(pCompLabels); pCompLabels=NULL;
  }
  if(pCompChecks!=NULL){
    safefree(pCompChecks); pCompChecks=NULL;
  }
}

// -----------------------------

static bool deskmf;

static void CB_MouseDown(s32 x,s32 y)
{
  deskmf=true;
}

static void CB_MouseMove(s32 x,s32 y)
{
}

static void CB_MouseUp(s32 x,s32 y)
{
  if(deskmf==false) return;
  deskmf=false;
  
  bool redraw=false;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_MouseUp(&pCompLabels[idx],x,y);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    if(ComponentCheck_MouseUp(&pCompChecks[idx],x,y)==true) redraw=true;
  }
  
  TiniImagePlugin *pi=&GlobalINI.ImagePlugin;
  if((s32)pi->VerticalPadding<0) pi->VerticalPadding=0;
  if(128<pi->VerticalPadding) pi->VerticalPadding=128;
  
  if(redraw==true){
    CompsReload();
    CompsRedraw(false);
  }
}

static void CB_Start(void)
{
  deskmf=false;
  
  DrawHelpScreen_Section("ImagePlugin");
  
  CompsInit();
  CompsReload();
  CompsRedraw(true);
}

static void CB_End(void)
{
  CompsFree();
}

void ProcImagePlugin2_SetCallBack(void)
{
  CallBack_Start=CB_Start;
  CallBack_End=CB_End;
  CallBack_MouseDown=CB_MouseDown;
  CallBack_MouseMove=CB_MouseMove;
  CallBack_MouseUp=CB_MouseUp;
}


