
#include <nds.h>

#include "../_const.h"
#include "../_console.h"
#include "../inifile.h"
#include "../memtool.h"
#include "../lang.h"

#include "global.h"
#include "rect.h"
#include "component.h"
#include "helpscr.h"

enum ECompLabels {ECLS_DelayCount,ECLS_DelayCountUnit,ECLS_DelayCountVal,
                  ECLS_RateCount,ECLS_RateCountUnit,ECLS_RateCountVal,
                  ECLSCount};
#define CompLabelsCount (ECLSCount)
static TComponentLabel *pCompLabels;

enum ECompChecks {ECCS_DelayCountMin,ECCS_DelayCountPlus,
                  ECCS_RateCountMin,ECCS_RateCountPlus,
                  ECCSCount};
#define CompChecksCount (ECCSCount)
static TComponentCheck *pCompChecks;

static void CompsReload(void)
{
  TiniKeyRepeat *pi=&GlobalINI.KeyRepeat;
  
  {
    static char val[8];
    snprintf(val,8,"%d",pi->DelayCount);
    pCompLabels[ECLS_DelayCountVal].pMsgUTF8=val;
  }
  {
    static char val[8];
    snprintf(val,8,"%d",pi->RateCount);
    pCompLabels[ECLS_RateCountVal].pMsgUTF8=val;
  }
}

static void CompsRedraw(bool FullDraw)
{
  if(FullDraw==true){
    CglCanvas *pcan=pScreenMainOverlay->pCanvas;
    
    u32 MenuBarHeight=20;
    pcan->SetColor(GlobalBGColor15);
    pcan->FillBox(0,MenuBarHeight,ScreenWidth,ScreenHeight-MenuBarHeight);
  }
  
  if(FullDraw==true){
    for(u32 idx=0;idx<CompLabelsCount;idx++){
      ComponentLabel_Draw(&pCompLabels[idx]);
    }
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
    }else{
    ComponentLabel_Draw(&pCompLabels[ECLS_DelayCountVal]);
    ComponentLabel_Draw(&pCompLabels[ECLS_RateCountVal]);
  }
}

static void CompsInit(void)
{
  pCompLabels=(TComponentLabel*)safemalloc(sizeof(TComponentLabel)*CompLabelsCount);
  pCompChecks=(TComponentCheck*)safemalloc(sizeof(TComponentCheck)*CompChecksCount);
  
  CglCanvas *pcan=pScreenMainOverlay->pCanvas;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_Init(&pCompLabels[idx],pcan);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    ComponentCheck_Init(&pCompChecks[idx],pcan);
  }
  
  TiniKeyRepeat *pi=&GlobalINI.KeyRepeat;
  pCompSectionName="KeyRepeat";
  
  s32 x,y;
  
  x=8; y=20;
  
  {
    pCompItemName="DelayCount";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_DelayCount],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_DelayCountMin],x+128,y,&pi->DelayCount,1);
    SetCompValueLabel(&pCompLabels[ECLS_DelayCountVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_DelayCountPlus],x+128+CompsCheckBoxSize+32,y,&pi->DelayCount,1);
    SetCompUnitLabel(&pCompLabels[ECLS_DelayCountUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="RateCount";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_RateCount],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_RateCountMin],x+128,y,&pi->RateCount,1);
    SetCompValueLabel(&pCompLabels[ECLS_RateCountVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_RateCountPlus],x+128+CompsCheckBoxSize+32,y,&pi->RateCount,1);
    SetCompUnitLabel(&pCompLabels[ECLS_RateCountUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
}

static void CompsFree(void)
{
  if(pCompLabels!=NULL){
    safefree(pCompLabels); pCompLabels=NULL;
  }
  if(pCompChecks!=NULL){
    safefree(pCompChecks); pCompChecks=NULL;
  }
}

// -----------------------------

static bool deskmf;

static void CB_MouseDown(s32 x,s32 y)
{
  deskmf=true;
}

static void CB_MouseMove(s32 x,s32 y)
{
}

static void CB_MouseUp(s32 x,s32 y)
{
  if(deskmf==false) return;
  deskmf=false;
  
  bool redraw=false;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_MouseUp(&pCompLabels[idx],x,y);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    if(ComponentCheck_MouseUp(&pCompChecks[idx],x,y)==true) redraw=true;
  }
  
  TiniKeyRepeat *pi=&GlobalINI.KeyRepeat;
  if((s32)pi->DelayCount<1) pi->DelayCount=1;
  if(60<pi->DelayCount) pi->DelayCount=60;
  if((s32)pi->RateCount<1) pi->RateCount=1;
  if(60<pi->RateCount) pi->RateCount=60;
  
  if(redraw==true){
    CompsReload();
    CompsRedraw(false);
  }
}

static void CB_Start(void)
{
  deskmf=false;
  
  DrawHelpScreen_Section("KeyRepeat");
  
  CompsInit();
  CompsReload();
  CompsRedraw(true);
}

static void CB_End(void)
{
  CompsFree();
}

void ProcKeyRepeat_SetCallBack(void)
{
  CallBack_Start=CB_Start;
  CallBack_End=CB_End;
  CallBack_MouseDown=CB_MouseDown;
  CallBack_MouseMove=CB_MouseMove;
  CallBack_MouseUp=CB_MouseUp;
}


