
#include <nds.h>

#include "../_const.h"
#include "../_console.h"
#include "../inifile.h"
#include "../memtool.h"
#include "../lang.h"

#include "global.h"
#include "rect.h"
#include "component.h"
#include "helpscr.h"

enum ECompLabels {ECLS_ClosedSholderButton,ECLS_FileMaxCount,ECLS_FileMaxCountUnit,ECLS_FileMaxCountVal,ECLSCount};
#define CompLabelsCount (ECLSCount)
static TComponentLabel *pCompLabels;

enum ECompChecks {ECCS_FileSelectSubScreen,
                  ECCS_ClosedSholderButton0,ECCS_ClosedSholderButton1,ECCS_ClosedSholderButton2,ECCS_ClosedSholderButton3,
                  ECCS_FullScreenOverlaySubScreen,
                  ECCS_FileMaxCountMin,ECCS_FileMaxCountPlus,
                  ECCSCount};
#define CompChecksCount (ECCSCount)
static TComponentCheck *pCompChecks;

static void CompsReload(void)
{
  TiniSystem *pi=&GlobalINI.System;
  
  pCompChecks[ECCS_ClosedSholderButton0].Checked=false;
  pCompChecks[ECCS_ClosedSholderButton1].Checked=false;
  pCompChecks[ECCS_ClosedSholderButton2].Checked=false;
  pCompChecks[ECCS_ClosedSholderButton3].Checked=false;
  switch(pi->ClosedSholderButton){
    case 0: pCompChecks[ECCS_ClosedSholderButton0].Checked=true; break;
    case 1: pCompChecks[ECCS_ClosedSholderButton1].Checked=true; break;
    case 2: pCompChecks[ECCS_ClosedSholderButton2].Checked=true; break;
    case 3: pCompChecks[ECCS_ClosedSholderButton3].Checked=true; break;
  }
  
  {
    static char val[8];
    snprintf(val,8,"%d",pi->FileMaxCount);
    pCompLabels[ECLS_FileMaxCountVal].pMsgUTF8=val;
  }
}

static void CompsRedraw(bool FullDraw)
{
  if(FullDraw==true){
    CglCanvas *pcan=pScreenMainOverlay->pCanvas;
    
    u32 MenuBarHeight=20;
    pcan->SetColor(GlobalBGColor15);
    pcan->FillBox(0,MenuBarHeight,ScreenWidth,ScreenHeight-MenuBarHeight);
  }
  
  if(FullDraw==true){
    for(u32 idx=0;idx<CompLabelsCount;idx++){
      ComponentLabel_Draw(&pCompLabels[idx]);
    }
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
    }else{
    ComponentLabel_Draw(&pCompLabels[ECLS_FileMaxCountVal]);
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
  }
}

static void CompsInit(void)
{
  pCompLabels=(TComponentLabel*)safemalloc(sizeof(TComponentLabel)*CompLabelsCount);
  pCompChecks=(TComponentCheck*)safemalloc(sizeof(TComponentCheck)*CompChecksCount);
  
  CglCanvas *pcan=pScreenMainOverlay->pCanvas;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_Init(&pCompLabels[idx],pcan);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    ComponentCheck_Init(&pCompChecks[idx],pcan);
  }
  
  TiniSystem *pi=&GlobalINI.System;
  pCompSectionName="System";
  
  s32 x,y;
  
  x=8; y=20;
  
  {
    pCompItemName="FileSelectSubScreen";
    y+=CompsLinePadding;
    SetCompCheck(&pCompChecks[ECCS_FileSelectSubScreen],x,y,&pi->FileSelectSubScreen);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="ClosedSholderButton";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_ClosedSholderButton],x,y);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_ClosedSholderButton0],x,y,&pi->ClosedSholderButton,0);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_ClosedSholderButton1],x,y,&pi->ClosedSholderButton,1);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_ClosedSholderButton2],x,y,&pi->ClosedSholderButton,2);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_ClosedSholderButton3],x,y,&pi->ClosedSholderButton,3);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="FullScreenOverlaySubScreen";
    y+=CompsLinePadding;
    SetCompCheck(&pCompChecks[ECCS_FullScreenOverlaySubScreen],x,y,&pi->FullScreenOverlaySubScreen);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="FileMaxCount";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_FileMaxCount],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_FileMaxCountMin],x+128,y,&pi->FileMaxCount,64);
    SetCompValueLabel(&pCompLabels[ECLS_FileMaxCountVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_FileMaxCountPlus],x+128+CompsCheckBoxSize+32,y,&pi->FileMaxCount,64);
    SetCompUnitLabel(&pCompLabels[ECLS_FileMaxCountUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
}

static void CompsFree(void)
{
  if(pCompLabels!=NULL){
    safefree(pCompLabels); pCompLabels=NULL;
  }
  if(pCompChecks!=NULL){
    safefree(pCompChecks); pCompChecks=NULL;
  }
}

// -----------------------------

static bool deskmf;

static void CB_MouseDown(s32 x,s32 y)
{
  deskmf=true;
}

static void CB_MouseMove(s32 x,s32 y)
{
}

static void CB_MouseUp(s32 x,s32 y)
{
  if(deskmf==false) return;
  deskmf=false;
  
  bool redraw=false;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_MouseUp(&pCompLabels[idx],x,y);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    if(ComponentCheck_MouseUp(&pCompChecks[idx],x,y)==true) redraw=true;
  }
  
  TiniSystem *pi=&GlobalINI.System;
  if((s32)pi->FileMaxCount<64) pi->FileMaxCount=64;
  if(768<pi->FileMaxCount) pi->FileMaxCount=768;
  
  if(redraw==true){
    CompsReload();
    CompsRedraw(false);
  }
}

static void CB_Start(void)
{
  deskmf=false;
  
  DrawHelpScreen_Section("System");
  
  CompsInit();
  CompsReload();
  CompsRedraw(true);
}

static void CB_End(void)
{
  CompsFree();
}

void ProcSystem1_SetCallBack(void)
{
  CallBack_Start=CB_Start;
  CallBack_End=CB_End;
  CallBack_MouseDown=CB_MouseDown;
  CallBack_MouseMove=CB_MouseMove;
  CallBack_MouseUp=CB_MouseUp;
}


