
#include <nds.h>

#include "../_const.h"
#include "../_console.h"
#include "../inifile.h"
#include "../memtool.h"
#include "../lang.h"

#include "global.h"
#include "rect.h"
#include "component.h"
#include "helpscr.h"

enum ECompLabels {ECLS_Mode,
                  ECLS_SelectItemAlpha,ECLS_SelectItemAlphaUnit,ECLS_SelectItemAlphaVal,
                  ECLS_UnselectItemAlpha,ECLS_UnselectItemAlphaUnit,ECLS_UnselectItemAlphaVal,
                  ECLSCount};
#define CompLabelsCount (ECLSCount)
static TComponentLabel *pCompLabels;

enum ECompChecks {ECCS_Mode0,ECCS_Mode1,ECCS_Mode2,ECCS_Mode3,
                  ECCS_SelectItemAlphaMin,ECCS_SelectItemAlphaPlus,
                  ECCS_UnselectItemAlphaMin,ECCS_UnselectItemAlphaPlus,
                  ECCSCount};
#define CompChecksCount (ECCSCount)
static TComponentCheck *pCompChecks;

static void CompsReload(void)
{
  TiniThumbnail *pi=&GlobalINI.Thumbnail;
  
  pCompChecks[ECCS_Mode0].Checked=false;
  pCompChecks[ECCS_Mode1].Checked=false;
  pCompChecks[ECCS_Mode2].Checked=false;
  pCompChecks[ECCS_Mode3].Checked=false;
  switch(pi->Mode){
    case 0: pCompChecks[ECCS_Mode0].Checked=true; break;
    case 1: pCompChecks[ECCS_Mode1].Checked=true; break;
    case 2: pCompChecks[ECCS_Mode2].Checked=true; break;
    case 3: pCompChecks[ECCS_Mode3].Checked=true; break;
  }
  
  {
    static char val[8];
    snprintf(val,8,"%d",pi->SelectItemAlpha);
    pCompLabels[ECLS_SelectItemAlphaVal].pMsgUTF8=val;
  }
  {
    static char val[8];
    snprintf(val,8,"%d",pi->UnselectItemAlpha);
    pCompLabels[ECLS_UnselectItemAlphaVal].pMsgUTF8=val;
  }
}

static void CompsRedraw(bool FullDraw)
{
  if(FullDraw==true){
    CglCanvas *pcan=pScreenMainOverlay->pCanvas;
    
    u32 MenuBarHeight=20;
    pcan->SetColor(GlobalBGColor15);
    pcan->FillBox(0,MenuBarHeight,ScreenWidth,ScreenHeight-MenuBarHeight);
  }
  
  if(FullDraw==true){
    for(u32 idx=0;idx<CompLabelsCount;idx++){
      ComponentLabel_Draw(&pCompLabels[idx]);
    }
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
    }else{
    ComponentLabel_Draw(&pCompLabels[ECLS_SelectItemAlphaVal]);
    ComponentLabel_Draw(&pCompLabels[ECLS_UnselectItemAlphaVal]);
    for(u32 idx=0;idx<CompChecksCount;idx++){
      ComponentCheck_Draw(&pCompChecks[idx]);
    }
  }
}

static void CompsInit(void)
{
  pCompLabels=(TComponentLabel*)safemalloc(sizeof(TComponentLabel)*CompLabelsCount);
  pCompChecks=(TComponentCheck*)safemalloc(sizeof(TComponentCheck)*CompChecksCount);
  
  CglCanvas *pcan=pScreenMainOverlay->pCanvas;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_Init(&pCompLabels[idx],pcan);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    ComponentCheck_Init(&pCompChecks[idx],pcan);
  }
  
  TiniThumbnail *pi=&GlobalINI.Thumbnail;
  pCompSectionName="Thumbnail";
  
  s32 x,y;
  
  x=8; y=20;
  
  {
    pCompItemName="Mode";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_Mode],x,y);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Mode0],x,y,&pi->Mode,0);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Mode1],x,y,&pi->Mode,1);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Mode2],x,y,&pi->Mode,2);
    y+=CompsLineHeight;
    SetCompRadio(&pCompChecks[ECCS_Mode3],x,y,&pi->Mode,3);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="SelectItemAlpha";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_SelectItemAlpha],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_SelectItemAlphaMin],x+128,y,&pi->SelectItemAlpha,1);
    SetCompValueLabel(&pCompLabels[ECLS_SelectItemAlphaVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_SelectItemAlphaPlus],x+128+CompsCheckBoxSize+32,y,&pi->SelectItemAlpha,1);
    SetCompUnitLabel(&pCompLabels[ECLS_SelectItemAlphaUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
  y+=CompsLineHeight;
  
  {
    pCompItemName="UnselectItemAlpha";
    y+=CompsLinePadding;
    SetCompNameLabel(&pCompLabels[ECLS_UnselectItemAlpha],x,y);
    SetCompMinCheck(&pCompChecks[ECCS_UnselectItemAlphaMin],x+128,y,&pi->UnselectItemAlpha,1);
    SetCompValueLabel(&pCompLabels[ECLS_UnselectItemAlphaVal],x+128+CompsCheckBoxSize,y);
    SetCompPlusCheck(&pCompChecks[ECCS_UnselectItemAlphaPlus],x+128+CompsCheckBoxSize+32,y,&pi->UnselectItemAlpha,1);
    SetCompUnitLabel(&pCompLabels[ECLS_UnselectItemAlphaUnit],x+128+CompsCheckBoxSize+32+CompsCheckBoxSize,y);
  }
  
}

static void CompsFree(void)
{
  if(pCompLabels!=NULL){
    safefree(pCompLabels); pCompLabels=NULL;
  }
  if(pCompChecks!=NULL){
    safefree(pCompChecks); pCompChecks=NULL;
  }
}

// -----------------------------

static bool deskmf;

static void CB_MouseDown(s32 x,s32 y)
{
  deskmf=true;
}

static void CB_MouseMove(s32 x,s32 y)
{
}

static void CB_MouseUp(s32 x,s32 y)
{
  if(deskmf==false) return;
  deskmf=false;
  
  bool redraw=false;
  
  for(u32 idx=0;idx<CompLabelsCount;idx++){
    ComponentLabel_MouseUp(&pCompLabels[idx],x,y);
  }
  for(u32 idx=0;idx<CompChecksCount;idx++){
    if(ComponentCheck_MouseUp(&pCompChecks[idx],x,y)==true) redraw=true;
  }
  
  TiniThumbnail *pi=&GlobalINI.Thumbnail;
  if((s32)pi->SelectItemAlpha<0) pi->SelectItemAlpha=0;
  if(16<pi->SelectItemAlpha) pi->SelectItemAlpha=16;
  if((s32)pi->UnselectItemAlpha<0) pi->UnselectItemAlpha=0;
  if(16<pi->UnselectItemAlpha) pi->UnselectItemAlpha=16;
  
  if(redraw==true){
    CompsReload();
    CompsRedraw(false);
  }
}

static void CB_Start(void)
{
  deskmf=false;
  
  DrawHelpScreen_Section("Thumbnail");
  
  CompsInit();
  CompsReload();
  CompsRedraw(true);
}

static void CB_End(void)
{
  CompsFree();
}

void ProcThumbnail_SetCallBack(void)
{
  CallBack_Start=CB_Start;
  CallBack_End=CB_End;
  CallBack_MouseDown=CB_MouseDown;
  CallBack_MouseMove=CB_MouseMove;
  CallBack_MouseUp=CB_MouseUp;
}


