
#include <NDS.h>

#include "../_const.h"
#include "../_console.h"

#include "../maindef.h"
#include "../memtool.h"
#include "../shell.h"

#include "../glib/glib.h"

#include "global.h"

#include "pagebtn.h"

CglB15 *pB15PageBtnL,*pB15PageBtnC,*pB15PageBtnR,*pB15PageBtnD,*pB15PageCloseBtn;
#define B15PageBtnPadding (4)

const char SectionItemName[EMI_Count][32]={
  "System1","System2","HiddenItem","KeyRepeat","BacklightTimeout","Boot",
  "Thumbnail","TextPlugin","ImagePlugin1","ImagePlugin2","GMEPlugin","EXIT"};

typedef struct {
  s32 Left,Width;
} TSectionItem;

static TSectionItem SectionItems[EMI_Count];
#define SectionItem_DrawWidth (ScreenWidth-16)
#define SectionItem_FullHeight (20)
#define SectionItem_TextTopOffset (4)

CglCanvas *pSectionItem_FullCanvas;

s32 PageOfsX;
u32 PageSelectIndex;

static void CreateFullCanvas(void)
{
  {
    u32 w=0;
    CglCanvas *pcan=new CglCanvas(NULL,1,1,pf15bit);
    for(u32 idx=0;idx<EMI_Count;idx++){
      u32 itemw=B15PageBtnPadding+pcan->GetTextWidthA(SectionItemName[idx]);
      SectionItems[idx].Left=w;
      SectionItems[idx].Width=itemw;
      w+=itemw;
    }
    delete pcan; pcan=NULL;
    w+=B15PageBtnPadding;
    pSectionItem_FullCanvas=new CglCanvas(NULL,w,SectionItem_FullHeight,pf15bit);
  }
}

static void RedrawFullCanvas(void)
{
  CglCanvas *pcan=pSectionItem_FullCanvas;
  
  pcan->SetColor(GlobalBGColor15);
  pcan->FillBox(0,0,pcan->GetWidth(),pcan->GetHeight());
  
  for(u32 idx=0;idx<EMI_Count;idx++){
    TSectionItem *psi=&SectionItems[idx];
    if(idx==PageSelectIndex){
      u32 x=psi->Left;
      pB15PageBtnL->BitBlt(pcan,x,0,4,SectionItem_FullHeight,0,0);
      for(u32 i=0;i<(u32)(psi->Width-B15PageBtnPadding+3)/4;i++){
        pB15PageBtnC->BitBlt(pcan,x+4+(i*4),0,4,SectionItem_FullHeight,0,0);
      }
      pB15PageBtnR->BitBlt(pcan,x+psi->Width,0,4,SectionItem_FullHeight,0,0);
      }else{
      u32 x=psi->Left;
      pB15PageBtnD->BitBlt(pcan,x+psi->Width,0,4,SectionItem_FullHeight,0,0);
    }
    pcan->TextOutA(psi->Left+B15PageBtnPadding,SectionItem_TextTopOffset,SectionItemName[idx]);
  }
}

void PageBtn_Init(u32 _PageIndex,u32 _PageOffset)
{
  {
    u8 *pb15;
    int b15size;
    Shell_ReadCustomFile("pagebtnl.b15",(void**)&pb15,&b15size);
    if(pb15!=NULL){
      pB15PageBtnL=new CglB15(pb15,b15size);
      safefree(pb15); pb15=NULL;
    }
  }
  {
    u8 *pb15;
    int b15size;
    Shell_ReadCustomFile("pagebtnc.b15",(void**)&pb15,&b15size);
    if(pb15!=NULL){
      pB15PageBtnC=new CglB15(pb15,b15size);
      safefree(pb15); pb15=NULL;
    }
  }
  {
    u8 *pb15;
    int b15size;
    Shell_ReadCustomFile("pagebtnr.b15",(void**)&pb15,&b15size);
    if(pb15!=NULL){
      pB15PageBtnR=new CglB15(pb15,b15size);
      safefree(pb15); pb15=NULL;
    }
  }
  {
    u8 *pb15;
    int b15size;
    Shell_ReadCustomFile("pagebtnd.b15",(void**)&pb15,&b15size);
    if(pb15!=NULL){
      pB15PageBtnD=new CglB15(pb15,b15size);
      safefree(pb15); pb15=NULL;
    }
  }
  {
    u8 *pb15;
    int b15size;
    Shell_ReadCustomFile("pageclosebtn.b15",(void**)&pb15,&b15size);
    if(pb15!=NULL){
      pB15PageCloseBtn=new CglB15(pb15,b15size);
      safefree(pb15); pb15=NULL;
    }
  }
  
  PageSelectIndex=_PageIndex;
  PageOfsX=_PageOffset;
  
  CreateFullCanvas();
}

void PageBtn_SetSelectIndex(u32 idx)
{
  PageSelectIndex=idx;
  RedrawFullCanvas();
}

void PageBtn_TransScreen(void)
{
  {
    CglCanvas *pcan=pSectionItem_FullCanvas;
    pcan->BitBlt(pScreenMainOverlay->pCanvas,0,0,SectionItem_DrawWidth,SectionItem_FullHeight,PageOfsX,0,false);
  }
  
  {
    CglB15 *pb15=pB15PageCloseBtn;
    pb15->BitBlt(pScreenMainOverlay->pCanvas,ScreenWidth-pb15->GetWidth(),0,pb15->GetWidth(),pb15->GetHeight(),0,0);
  }
}

u32 PageBtn_GetPageIndex(void)
{
  return(PageSelectIndex);
}

u32 PageBtn_GetPageOffset(void)
{
  return((u32)PageOfsX);
}

static s32 mx,my;
static bool mf=false;
static s32 selidx;
static bool PressCloseBtn;

void PageBtn_MouseDown(s32 x,s32 y)
{
  mf=true;
  mx=x;
  my=y;
  
  selidx=-1;
  PressCloseBtn=false;
  
  if((SectionItem_DrawWidth-16)<=x){
    PressCloseBtn=true;
    return;
  }
  
  x+=PageOfsX;
  
  for(u32 idx=0;idx<EMI_Count;idx++){
    s32 l=SectionItems[idx].Left;
    s32 w=SectionItems[idx].Width;
    if((l<=x)&&(x<(l+w))){
      if(idx!=PageSelectIndex) selidx=idx;
    }
  }
}

void PageBtn_MouseMove(s32 x,s32 y)
{
  if(mf==false) return;
  
  if(PressCloseBtn==true) return;
  
  s32 v=mx-x;
  mx=x;
  
  PageOfsX+=v;
  if((pSectionItem_FullCanvas->GetWidth()-SectionItem_DrawWidth)<PageOfsX) PageOfsX=pSectionItem_FullCanvas->GetWidth()-SectionItem_DrawWidth;
  if(PageOfsX<0) PageOfsX=0;
  
  PageBtn_TransScreen();
}

void PageBtn_MouseUp(s32 x,s32 y)
{
  if(mf==false) return;
  mf=false;
  
  if(PressCloseBtn==true){
    if((SectionItem_DrawWidth-16)<=x) NextMenuPage=EMI_Exit;
    }else{
    if(selidx!=-1) NextMenuPage=(ESectionItem)selidx;
  }
  
  selidx=-1;
  PressCloseBtn=false;
}

