
#ifndef inifile_h
#define inifile_h

#include "rect.h"

enum EClosedSholderButton {ECSB_Disabled=0,ECSB_Flexible=1,ECSB_AlwaysDisabled=2,ECSB_Enabled=3};
enum EScreenFlip {ESF_Normal=0,ESF_Flip=1,ESF_VFlip=2,WFS_HFlip=3};
enum EiniSystemWhenPanelClose {EISWPC_BacklightOff=0,EISWPC_DSPowerOff=1,EISWPC_PlayShutdownSound=2};

enum EiniSystemMusicNext {EISMN_Stop=0,EISMN_Repeat=1,EISMN_NormalLoop=2,EISMN_NormalPOff=3,EISMN_ShuffleLoop=4,EISMN_ShufflePOff=5,EISMN_PowerOff=6,EISMN_Count=7};

typedef struct {
  u32 PageIndex,PageOffset,ExitProcIndex;
} TiniCustom;

enum EiniStartButtonExec {EISBE_ShowDebugLog=0,EISBE_SoftReset=1};

typedef struct {
  bool FileSelectSubScreen;
  EClosedSholderButton ClosedSholderButton;
  EiniSystemMusicNext MusicNext;
  int SoundVolume;
  bool FullScreenOverlaySubScreen;
  u32 FileMaxCount;
  u32 NDSLiteDefaultBrightness;
  EiniSystemWhenPanelClose WhenPanelClose;
  EScreenFlip TopScreenFlip;
  bool ResumeUsingWhileMusicPlaying;
  bool LockLRButton;
  EiniStartButtonExec StartButtonExec;
  bool PrivateFunction;
} TiniSystem;

typedef struct {
    u32 ROM1stAccessCycleControl;
    u32 ROM2stAccessCycleControl;
} TiniForSuperCard;

typedef struct {
  bool Attribute_Archive;
  bool Attribute_Hidden;
  bool Attribute_System;
  bool Attribute_Readonly;
  bool Path_Shell;
  bool Path_Moonshl;
  bool File_Thumbnail;
} TiniHiddenItem;

typedef struct {
  u32 DelayCount;
  u32 RateCount;
} TiniKeyRepeat;

typedef struct {
  u32 WhenStandby;
  u32 WhenPicture;
  u32 WhenText;
  u32 WhenSound;
} TiniBacklightTimeout;

enum EiniBootStartupSound {EIBSS_Off=0,EIBSS_Startupmp3=1,EIBSS_AutoPlay=2};

typedef struct {
  bool hiddenAboutWindow;
  bool hiddenHelpWindow;
  bool hiddenDateTimeWindow;
  u32 StartupSound;
} TiniBoot;

enum EiniThumbnailMode {EITM_Off=0,EITM_ThumbSelector=1,EITM_FullscreenPreview=2,EITM_Both=3};

typedef struct {
  EiniThumbnailMode Mode;
  u32 SelectItemAlpha;
  u32 UnselectItemAlpha;
} TiniThumbnail;

enum EiniTextPluginSelectDisplay {EITPSD_Bottom=0,EITPSD_Top=1,EITPSD_DualScreen=2};
enum EiniTextPluginTextEncode {EITPTE_Local=0,EITPTE_UTF8=1,EITPTE_UTF16LE=2,EITPTE_UTF16BE=3};

typedef struct {
  u32 SpacePixel;
  u32 ScrollType;
  EiniTextPluginSelectDisplay SelectDisplay;
  EiniTextPluginTextEncode TextEncode;
} TiniTextPlugin;

typedef struct {
  bool CrashMeProtection;
  bool NDSROMBootFunctionWithEZ4SD;
} TiniNDSROMPlugin;

enum EDPGDithering {EDPGD_NoCnt=0,EDPGD_HalfCnt=1,EDPGD_QuatCnt=2,EDPGD_HeavyCnt=3};

typedef struct {
  EDPGDithering Dithering;
  bool GOPSkip;
  bool AdaptiveDelayControl;
} TiniDPGPlugin;

#include "imgcalc.h"

typedef struct {
  u32 GUITimeOutSec;
  EImageMode ImageMode;
  bool DrawFileInfo;
  u32 VerticalPadding;
  u32 Interpolation;
} TiniImagePlugin;

typedef struct {
  bool Enabled;
  bool RTC24Hour;
  char FormatDate[32+1];
} TiniClockPlugin;

typedef struct {
  s32 x,y,w,h;
  bool DisabledCloseButton;
} TiniOverrideWindowRectBody;

typedef struct {
  TiniOverrideWindowRectBody About,Help,PlayControl,FileSelect,DateTime,PicView,TextView,Progress,SetNext,FileInfo,SystemIcon;
} TiniOverrideWindowRect;

typedef struct {
  bool SC_EnabledDRAM;
  u32 EZ4_PSRAMSizeMByte;
} TiniAdapterConfig;

enum EGMEPluginSimpleLPF {EGMESimpleLPF_None=0,EGMESimpleLPF_Lite=1,EGMESimpleLPF_Heavy=2};

typedef struct {
  u32 ReverbLevel;
  EGMEPluginSimpleLPF SimpleLPF;
  u32 DefaultLengthSec;
  u32 HES_MaxTrackNumber;
} TiniGMEPlugin;

typedef struct {
  TiniCustom Custom;
  TiniSystem System;
  TiniForSuperCard ForSuperCard;
  TiniHiddenItem HiddenItem;
  TiniKeyRepeat KeyRepeat;
  TiniBacklightTimeout BacklightTimeout;
  TiniBoot Boot;
  TiniThumbnail Thumbnail;
  TiniTextPlugin TextPlugin;
  TiniNDSROMPlugin NDSROMPlugin;
  TiniDPGPlugin DPGPlugin;
  TiniImagePlugin ImagePlugin;
  TiniClockPlugin ClockPlugin;
  TiniOverrideWindowRect OverrideWindowRect;
  TiniGMEPlugin GMEPlugin;
  TiniAdapterConfig AdapterConfig;
} TGlobalINI;

extern TGlobalINI GlobalINI;

extern void InitINI(void);
extern void InitINI_Skin(void);

extern void LoadINI_moonshl_ini(void);
extern void LoadINI_skin_ini(void);
extern void LoadINIFromCurrent(char *inifn);

extern u8 INI_MD5Data[16];

#endif


