
        AREA   INIT946TCM, CODE, READONLY   ; name this block of code

__itcm_start equ 0x00000000
__dtcm_start equ 0x00800000

PAGE_4K EQU 2_01011 << 1
PAGE_8K EQU 2_01100 << 1
PAGE_16K EQU 2_01101 << 1
PAGE_32K EQU 2_01110 << 1
PAGE_64K EQU 2_00111 << 1
PAGE_128K EQU 2_10000 << 1
PAGE_256K EQU 2_10001 << 1
PAGE_512K EQU 2_10010 << 1
PAGE_1M EQU 2_10011 << 1
PAGE_2M EQU 2_10100 << 1
PAGE_4M EQU 2_10101 << 1
PAGE_8M EQU 2_10110 << 1
PAGE_16M EQU 2_10111 << 1
PAGE_32M EQU 2_11000 << 1
PAGE_64M EQU 2_11001 << 1
PAGE_128M EQU 2_11010 << 1
PAGE_256M EQU 2_11011 << 1
PAGE_512M EQU 2_11100 << 1
PAGE_1G EQU 2_11101 << 1
PAGE_2G EQU 2_11110 << 1
PAGE_4G EQU 2_11111 << 1

ITCM_LOAD EQU 1<<19
ITCM_ENABLE EQU 1<<18
DTCM_LOAD EQU 1<<17
DTCM_ENABLE EQU 1<<16
DISABLE_TBIT EQU 1<<15
ROUND_ROBIT EQU 1<<14
ALT_VECTORS EQU 1<<13
ICACHE_ENABLE EQU 1<<12
BIG_ENDIAN EQU 1<<7
DCACHE_ENABLE EQU 1<<2
PROTECT_ENABLE EQU 1<<0

;---------------------------------------------------------------------------------
; _start
;---------------------------------------------------------------------------------
	ldr	r0, =0x04000208			; REG_IME = 0;
	mov r1, #0
	strh	r1, [r0]

;---------------------------------------------------------------------------------
; turn the power on for M3
;---------------------------------------------------------------------------------
	ldr	r0, =0x04000304 ; REG_POWER
	ldr r1, =0x8203 ; POWER_LCD(0) POWER_2D_A(1) POWER_2D_B(9) POWER_SWAP_LCDS(15)
	strh    r1, [r0]

	ldr	r1, =0x00002078			; disable TCM and protection unit
	mcr	p15, 0, r1, c1, c0

;---------------------------------------------------------------------------------
; Protection Unit Setup added by Sasq
;---------------------------------------------------------------------------------
	; Disable cache
	mov	r0, #0
	mcr	p15, 0, r0, c7, c5, 0		; Instruction cache
	mcr	p15, 0, r0, c7, c6, 0		; Data cache

	; Wait for write buffer to empty 
	mcr	p15, 0, r0, c7, c10, 4

	ldr	r0, =(__dtcm_start | 0x0a)
	mcr	p15, 0, r0, c9, c1,0		; DTCM base = __dtcm_start, size = 16 KB

	;ITCM address is fixed.
	ldr	r0, =(__itcm_start | 0x0c)
	mcr	p15, 0, r0, c9, c1,1		; ITCM base = __itcm_start, size = 32 KB
	
;---------------------------------------------------------------------------------
; Setup memory regions similar to Release Version
;	this code currently breaks dualis
;---------------------------------------------------------------------------------

	;-------------------------------------------------------------------------
	; Region 0 - IO registers
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_64M | 0x04000000 | 1)	
	mcr	p15, 0, r0, c6, c0, 0

	;-------------------------------------------------------------------------
	; Region 1 - Main Memory
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_4M | 0x02000000 | 1)	
	mcr	p15, 0, r0, c6, c1, 0

	;-------------------------------------------------------------------------
	; Region 2 - iwram
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_32K | 0x037F8000 | 1)	
	mcr	p15, 0, r0, c6, c2, 0

	;-------------------------------------------------------------------------
	; Region 3 - DS Accessory (GBA Cart)
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_128M | 0x08000000 | 1)	
	mcr	p15, 0, r0, c6, c3, 0

	;-------------------------------------------------------------------------
	; Region 4 - DTCM
	;-------------------------------------------------------------------------
	ldr	r0,=(PAGE_16K | __dtcm_start | 1)
	mcr	p15, 0, r0, c6, c4, 0

	;-------------------------------------------------------------------------
	; Region 5 - ITCM
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_32K | __itcm_start | 1)
	mcr	p15, 0, r0, c6, c5, 0

	;-------------------------------------------------------------------------
	; Region 6 - System ROM
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_32K | 0xFFFF0000 | 1)	
	mcr	p15, 0, r0, c6, c6, 0

	;-------------------------------------------------------------------------
	; Region 7 - non cacheable main ram
	;-------------------------------------------------------------------------
	ldr	r0,=( PAGE_4M  | 0x02400000 | 1)	
	mcr	p15, 0, r0, c6, c7, 0

	;-------------------------------------------------------------------------
	; Write buffer enable
	;-------------------------------------------------------------------------
	ldr	r0,=2_00000110
	;ldr	r0,=2_00000111 ; region0 IO registers enabled.
	mcr	p15, 0, r0, c3, c0, 0

	;-------------------------------------------------------------------------
	; DCache & ICache enable
	;-------------------------------------------------------------------------
	ldr	r0,=2_01000010
	mcr	p15, 0, r0, c2, c0, 0
	mcr	p15, 0, r0, c2, c0, 1

	;-------------------------------------------------------------------------
	; IAccess
	;-------------------------------------------------------------------------
	ldr	r0,=0x36636333
	;ldr	r0,=0x36333333 ; region3,5 to RW
	mcr	p15, 0, r0, c5, c0, 3

	;-------------------------------------------------------------------------
	; DAccess
	;------------------------------------------------------------------------
	ldr	r0,=0x36333333
	mcr     p15, 0, r0, c5, c0, 2

	;-------------------------------------------------------------------------
	; Enable ICache, DCache, ITCM & DTCM
	;-------------------------------------------------------------------------
	mrc	p15, 0, r0, c1, c0, 0
	ldr r1,= ~(ITCM_ENABLE | DTCM_ENABLE | ICACHE_ENABLE | DCACHE_ENABLE | PROTECT_ENABLE)
	and r0,r0,r1
	
	;ldr	r1,= 0
	ldr	r1,= ITCM_ENABLE | DTCM_ENABLE | ICACHE_ENABLE | DCACHE_ENABLE | PROTECT_ENABLE
	;ldr	r1,= ITCM_ENABLE | DTCM_ENABLE | PROTECT_ENABLE
	;ldr	r1,= ITCM_ENABLE | DTCM_ENABLE
	orr	r0,r0,r1
	mcr	p15, 0, r0, c1, c0, 0
 
	mov	r0, #0x13		; Switch to SVC Mode
	msr	cpsr_cxsf, r0
	ldr	sp, =0x00803fe0		; Set SVC stack

	mov	r0, #0x12		; Switch to IRQ Mode
	msr	cpsr_cxsf, r0
	ldr	sp, =0x00803fa0		; Set IRQ stack

	mov	r0, #0x1F		; Switch to System Mode
	msr	cpsr_cxsf, r0
	ldr	sp, =0x00803e00		; Set user stack (not use this parametor. see init.s)
        
    IMPORT  __main                     ; import label to __main
    LDR pc,=__main                     ; branch to C Library entry 
	
	END
	
