
static s32 FS_TopIndex;
s32 FS_CursorIndex;
static s32 FS_IndexCount;
s32 FS_PlayIndex;
static s32 FS_DispCount;
static bool FS_RedrawFlag[32]; // overspec.
static bool FS_ExculsiveAllRedrawFlag;

static int FileHandle=0;
static int FileHandle1=0,FileHandle2=0;

char forResume_PlayPathName[forResume_PlayPathNameLen];
char forResume_PlayFileAliasName[forResume_PlayFileAliasNameLen];
UnicodeChar forResume_PlayFileUnicodeName[forResume_PlayFileUnicodeNameLen];

static void FS_Redraw_SetAll(void)
{ cwl();
  for(u32 idx=0;idx<(u32)FS_DispCount;idx++){ cwl();
    FS_RedrawFlag[idx]=true;
  }
  FS_ExculsiveAllRedrawFlag=true;
}

static void FS_Redraw_SetPlayIndex(void)
{ cwl();
  if(FS_PlayIndex==-1) return;
  
  s32 idx=FS_PlayIndex-FS_TopIndex;
  if((idx<0)||(FS_DispCount<=idx)) return;
  FS_RedrawFlag[idx]=true;
}

static void FS_Redraw_SetCursorIndex(void)
{ cwl();
  if(FS_CursorIndex==-1) return;
  
  s32 idx=FS_CursorIndex-FS_TopIndex;
  if((idx<0)||(FS_DispCount<=idx)) return;
  FS_RedrawFlag[idx]=true;
}

static void FS_Redraw_SetTopIndex(void)
{ cwl();
  FS_Redraw_SetAll();
}

void FS_ChangePath(const char *TargetPathName)
{ cwl();
  FileSys_ChangePath(TargetPathName);
  
  thumbChangePath();
  if(thumbExists()==true){
    FS_DispCount=9;
    }else{
    FS_DispCount=MWin_GetClientHeight(WM_FileSelect)/FontProHeight;
  }
  
  FS_TopIndex=0;
  FS_CursorIndex=0;
  FS_IndexCount=FileSys_GetFileCount();
  FS_PlayIndex=-1;
  FS_Redraw_SetAll();
  
  { cwl();
    s32 lastidx=FileSys_GetLastPathItemIndex();
    if(lastidx!=-1){ cwl();
      FS_SetCursor(lastidx);
    }
  }
  
  s32 idxcnt=FS_IndexCount;
  if(thumbExists()==true) idxcnt=(idxcnt+2)/3*3;
  MWin_SetSBarV(WM_FileSelect,idxcnt,FS_DispCount,FS_TopIndex);
  
  { cwl();
    char str[256];
    sprintf(str,"%s %dfiles.",FileSys_GetPathName(),FileSys_GetPureFileCount());
    MWin_SetWindowTitle(WM_FileSelect,str);
  }
  
  MWin_RefreshWindow(WM_FileSelect);
  MWin_TransWindow(WM_FileSelect);
}

static void FS_ExecutePath(u32 FileIndex)
{ cwl();
  char *TargetPathName=FileSys_GetAlias(FileIndex);
  
  FS_ChangePath(TargetPathName);
  Setting_Backup_LastPath(FileSys_GetPathName());
}

void FS_ExecuteStop(void)
{ cwl();
  thumbReturnImage();
  
  if(ExecMode==EM_Text){
    Bookmark_CurrentResumeBackup();
    Bookmark_Save();
    Bookmark_End();
  }
  
  if(ExecMode!=EM_None) LastExecMode=ExecMode;
  ExecMode=EM_None;
  
  MWin_PlayControl_SetWindowTitleW(NULL);
  
  strpcmStop();
  
  for(u32 idx=0;idx<SoundEffectPluginCount;idx++){
    if(pSoundEffectPluginBody[idx]!=NULL){
      DLLList_FreeSoundEffectPlugin(idx);
      safefree(pSoundEffectPluginBody[idx]); pSoundEffectPluginBody[idx]=NULL;
    }
  }
  
  if(pPluginBody!=NULL){
    DLLList_FreePlugin(pPluginBody);
    safefree(pPluginBody); pPluginBody=NULL;
  }
  
  FreeFPK();
  FreeText();
  FreeDPG();
  FreeGME();
  
  if(FileHandle!=0){ cwl();
    FileSys_fclose(FileHandle);
    FileHandle=0;
  }
  
  if(FileHandle1!=0){ cwl();
    FileSys_fclose(FileHandle1);
    FileHandle1=0;
  }
  
  if(FileHandle2!=0){ cwl();
    FileSys_fclose(FileHandle2);
    FileHandle2=0;
  }
  
  bool RefreshFlag=false;
  
  if(MWin_GetVisible(WM_PlayControl)==true){ cwl();
    MWin_SetVisible(WM_PlayControl,false);
    RefreshFlag=true;
  }
  MWin_FreeMem(WM_PlayControl);
  
  PlugInfo_InitCustomMessage(false);
  if(MWin_GetVisible(WM_FileInfo)==true){ cwl();
    MWin_SetVisible(WM_FileInfo,false);
    RefreshFlag=true;
  }
  MWin_FreeMem(WM_FileInfo);
  
  if(MWin_GetVisible(WM_PicView)==true){ cwl();
    MWin_SetVisible(WM_PicView,false);
    RefreshFlag=true;
  }
  MWin_FreeMem(WM_PicView);
  
  if(MWin_GetVisible(WM_TextView)==true){ cwl();
    MWin_SetVisible(WM_TextView,false);
    RefreshFlag=true;
  }
  MWin_FreeMem(WM_TextView);
  
  if(MWin_GetVisible(WM_BinView)==true){ cwl();
    MWin_SetVisible(WM_BinView,false);
    RefreshFlag=true;
  }
  MWin_FreeMem(WM_BinView);
  
  if(RefreshFlag==true){ cwl();
    MWin_SetActiveTopMost();
    MWin_RefreshScreenMask();
    MWin_AllRefresh();
  }
  
  switch(LastExecMode){
    case EM_DPG: case EM_None: case EM_MP3Boot: case EM_NDSROM: case EM_FPK: case EM_MSPImage: case EM_Text: case EM_BinView: {
      MWin_SetVideoMode(ESMM_Normal);
      MWin_SetVideoOverlaySize(ScreenWidth,ScreenHeight,0);
      MWin_SetVideoOverlay(false);
      MWin_RefreshVideoFullScreen(false);
      
      if(GlobalINI.System.FileSelectSubScreen==false){
        // exclusive reset
        BacklightOffVsyncCount=1;
        PluginBodyClock_Execute=true;
        PluginBodyClock_RequestRefresh=true;
      }
//      videoSetModeSub_SetShowLog(true);
    } break;
    case EM_MSPSound: case EM_GME: {
    } break;
  }
  
  PrintFreeMem();
}

static bool FS_FileOpen(u32 FileIndex)
{ cwl();
  FileHandle=FileSys_fopen(FileIndex);
  
  if(FileHandle==0) return(false);
  
  return(true);
}

static bool FS_FileOpen2(u32 FileIndex)
{ cwl();
  FileHandle1=FileSys_fopen(FileIndex);
  
  if(FileHandle1==0) return(false);
  
  FileHandle2=FileSys_fopen(FileIndex);
  
  if(FileHandle2==0) return(false);
  
  return(true);
}

static EExecMode FS_GetExecMode(u32 FileIndex)
{ cwl();
  if(FileSys_GetFileType(FileIndex)!=FT_File) return(EM_None);
  
  char ext[8];
  
  FileSys_GetFileExt(FileIndex,ext);
  
  if(ext[0]==0) return(EM_None);
  
  {
    const char *pmsefn=MSEList_GetPluginFilename(ext);
    if(pmsefn!=NULL){
      MSE_Exec(pmsefn);
      while(1);
    }
  }
  
#ifdef USEGME
  if(isStrEqual(ext,".NSF")==true) return(EM_GME);
  if(isStrEqual(ext,".GBS")==true) return(EM_GME);
  if(isStrEqual(ext,".HES")==true) return(EM_GME);
  if(isStrEqual(ext,".AY")==true) return(EM_GME);
  if(isStrEqual(ext,".SAP")==true) return(EM_GME);
//  if(isStrEqual(ext,".SPC")==true) return(EM_GME);
  if(isStrEqual(ext,".KSS")==true) return(EM_GME);
#endif
  
  {
    char fn[PluginFilenameMax];
    
    EPluginType EPT=DLLList_GetPluginFilename(ext,fn);
    
    switch(EPT){
      case EPT_None: break;
      case EPT_Image: return(EM_MSPImage); break;
      case EPT_Sound: return(EM_MSPSound); break;
      case EPT_Clock: {
        _consolePrint("Error:Clock plugin is not execute for file.\n");
      } break;
      case EPT_SoundEffect: {
        _consolePrint("Error:SoundEffect plugin is not execute for file.\n");
      } break;
    }
  }
  
#ifdef USENDSROM
  if(isStrEqual(ext,".NDS")==true) return(EM_NDSROM);
  if(isStrEqual(ext,".MSE")==true) return(EM_NDSROM);
#endif
  
  if(isStrEqual(ext,".FPK")==true) return(EM_FPK);
  
  if((isStrEqual(ext,".TXT")==true)||(isStrEqual(ext,".DOC")==true)||(isStrEqual(ext,".INI")==true)) return(EM_Text);
  if((isStrEqual(ext,".C")==true)||(isStrEqual(ext,".CPP")==true)||(isStrEqual(ext,".H")==true)) return(EM_Text);
  if((isStrEqual(ext,".MML")==true)||(isStrEqual(ext,".FRM")==true)) return(EM_Text);
  if(isStrEqual(ext,".HTM")==true) return(EM_Text);
  
  if((isStrEqual(ext,".BIN")==true)||(isStrEqual(ext,".SAV")==true)) return(EM_BinView);
  
#ifdef USEDPG
  if(isStrEqual(ext,".DPG")==true) return(EM_DPG);
#endif
  
  return(EM_None);
}

static void FS_ExecuteFile_SetWindowTitle(u32 WinIndex,u32 FileIndex,char *WindowTitle)
{ cwl();
  char ttlbuf[512];
  
  sprintf(ttlbuf,"%s %s",FileSys_GetAlias(FileIndex),WindowTitle);
  MWin_SetWindowTitle(WinIndex,ttlbuf);
  
  MWin_PlayControl_SetWindowTitleW(FileSys_GetFilename(FileIndex));
}

static void FS_ExecuteFile_InitPlayControlWindow(u32 FileIndex)
{ cwl();
  FS_ExecuteFile_SetWindowTitle(WM_PlayControl,FileIndex,"PlayControl...");
  
  MWin_SetVisible(WM_PlayControl,true);
  u32 WinIndex=MWin_SetActive(WM_PlayControl);
  MWin_RefreshScreenMask();
  if(WinIndex!=(u32)-1) MWin_RefreshWindow(WinIndex);
  MWin_RefreshWindow(WM_PlayControl);
}

static void FS_ExecuteFile_InitFileInfoWindow(void)
{ cwl();
  if(PlugInfo_GetShowFlag()==false) return;
  
  TWinBody *pwb=MWin_GetWinBody(WM_FileInfo);
  if(pwb==NULL) return;
  
  MWin_SetSBarV(WM_FileInfo,PlugInfo_GetInfoIndexCount(),pwb->ClientRect.h/FontProHeight,0);
  
  MWin_SetVisible(WM_FileInfo,true);
  u32 WinIndex=MWin_SetActive(WM_FileInfo);
  MWin_RefreshScreenMask();
  if(WinIndex!=(u32)-1) MWin_RefreshWindow(WinIndex);
  MWin_RefreshWindow(WM_FileInfo);
}

static void FS_ExecuteFile_PicView_DrawFileInfo(void)
{
  TWinBody *pwb=MWin_GetWinBody(WM_PicView);
  if(pwb==NULL) return;
  
  u16 *pBuf=&pwb->WinBuf[0];
  s32 BufWidth=pwb->Rect.w;
  
  {
    for(int y=0;y<pwb->ClientRect.h;y++){
      MemSet16DMA3(0,&pBuf[pwb->ClientRect.x+((pwb->ClientRect.y+y)*BufWidth)],pwb->ClientRect.w*2);
    }
  }
  
  if(GlobalINI.ImagePlugin.DrawFileInfo==false) return;
  
  if(PlugInfo_GetShowFlag()==false) return;
  
  int InfoCount=PlugInfo_GetInfoIndexCount();
  int LineCount=(pwb->ClientRect.h/FontProHeight)-1;
  int TopIndex=LineCount-InfoCount;
  
  if(TopIndex<0) TopIndex=0;
  
  u16 colp=RGB15(31,31,31)|BIT15;
  u16 coln=RGB15(0,0,0)|BIT15;
  
  s32 wLimit=pwb->ClientRect.x+pwb->ClientRect.w;
  u32 ofsx=pwb->ClientRect.x+4;
  u32 ofsy=pwb->ClientRect.y+0;
  
  for(int idx=0;idx<InfoCount;idx++){
    int drawidx=TopIndex+idx;
    if(drawidx<LineCount){
      {
        char str[256];
        str[0]=0;
        if(PlugInfo_GetInfoStrL(idx,str,256)==true){
          VRAM_WriteStringRectL(pBuf,BufWidth,wLimit,ofsx+1,ofsy+(drawidx*FontProHeight)+1,str,coln);
          VRAM_WriteStringRectL(pBuf,BufWidth,wLimit,ofsx,ofsy+(drawidx*FontProHeight),str,colp);
        }
      }
      {
        UnicodeChar str[256];
        str[0]=0;
        if(PlugInfo_GetInfoStrW(idx,str,256)==true){
          VRAM_WriteStringRectW(pBuf,BufWidth,wLimit,ofsx+1,ofsy+(drawidx*FontProHeight)+1,str,coln);
          VRAM_WriteStringRectW(pBuf,BufWidth,wLimit,ofsx,ofsy+(drawidx*FontProHeight),str,colp);
        }
      }
      {
        char str[256*2];
        str[0]=0;
        if(PlugInfo_GetInfoStrUTF8(idx,str,256)==true){
          VRAM_WriteStringRectUTF8(pBuf,BufWidth,wLimit,ofsx+1,ofsy+(drawidx*FontProHeight)+1,str,coln);
          VRAM_WriteStringRectUTF8(pBuf,BufWidth,wLimit,ofsx,ofsy+(drawidx*FontProHeight),str,colp);
        }
      }
    }
  }
}

static void FS_ExecuteFile_LoadFail(void)
{
  ExecMode=EM_None;
  LastExecMode=EM_None;
}

static EstrpcmFormat FS_ExecuteFile_GetOversamplingFactorFromSampleRate(u32 SampleRate)
{
  _consolePrintf("SampleRate=%dHz\n",SampleRate);
  
  EstrpcmFormat SPF;
  
  if(SampleRate==(32768/4)){
    SPF=SPF_PCMx4;
    }else{
    if(SampleRate==(32768/2)){
      SPF=SPF_PCMx2;
      }else{
      if(SampleRate==(32768/1)){
        SPF=SPF_PCMx1;
        }else{
        if(SampleRate<=48000){
          SPF=SPF_PCMx4;
          }else{
          if(SampleRate<=96000){
            SPF=SPF_PCMx2;
            }else{
            SPF=SPF_PCMx1;
          }
        }
      }
    }
  }
  
  return(SPF);
}

static void FS_ExecuteFile(u32 FileIndex,EExecMode OverrideExecMode)
{ cwl();
  
#ifdef ExecuteShowDebug
  videoSetModeSub_SetShowLog(true);
#endif
  
  _consoleClear();
  _consolePrintSet(0,2);
  
  FS_ExecuteStop();
  
  if(OverrideExecMode==EM_None){
    if(FileSys_GetFileType(FileIndex)==FT_File){
      if(strcmp(FileSys_GetAlias(FileIndex),"SKIN.INI")==0){
        InitINI_Skin();
        LoadINIFromCurrent("skin.ini");
        MWin_ReloadSkinFromCurrent();
        FS_Redraw_SetAll();
        MWin_RefreshScreenMask();
        MWin_ClearDesktop();
        MWin_AllRefresh();
        return;
      }
    }
  }
  
  PrintFreeMem();
  
  {
    strncpy(forResume_PlayPathName,FileSys_GetPathName(),forResume_PlayPathNameLen);
    strncpy(forResume_PlayFileAliasName,FileSys_GetAlias(FileIndex),forResume_PlayFileAliasNameLen);
    Unicode_StrCopy(FileSys_GetFilename(FileIndex),forResume_PlayFileUnicodeName);
  }
  
  if(OverrideExecMode!=EM_None){
    ExecMode=OverrideExecMode;
    }else{
    ExecMode=FS_GetExecMode(FileIndex);
  }
  
  if(ExecMode==EM_None){ cwl();
    FS_ExecuteFile_LoadFail();
    
    char ext[8];
    
    FileSys_GetFileExt(FileIndex,ext);
    PlugInfo_InitCustomMessage(true);
    snprintf(PlugInfo_CustomMessage0,64,Lang_GetInternalIndex(ELI_NotSupportFileExt,0),ext);
    snprintf(PlugInfo_CustomMessage1,64,Lang_GetInternalIndex(ELI_NotSupportFileExt,1),ext);
    snprintf(PlugInfo_CustomMessage2,64,Lang_GetInternalIndex(ELI_NotSupportFileExt,2),ext);
    FS_ExecuteFile_InitFileInfoWindow();
    return;
  }
  
#ifdef USENDSROM
  if(ExecMode==EM_NDSROM){ cwl();
    static char Filename[PathnameMaxLength];
    char *pPath=FileSys_GetPathName();
    if(strlen(pPath)<2) pPath="";
    snprintf(Filename,PathnameMaxLength,"%s/%s",pPath,FileSys_GetAlias(FileIndex));
    if(BootNDSROM(Filename)==true){ cwl();
      while(1);
      }else{ cwl();
      FS_ExecuteFile_LoadFail();
      PlugInfo_InitCustomMessage(true);
      snprintf(PlugInfo_CustomMessage0,64,"not support adapter?");
      FS_ExecuteFile_InitFileInfoWindow();
    }
    return;
  }
#endif
  
  if(ExecMode==EM_FPK){ cwl();
    MWin_AllocMem(WM_PlayControl);
    if(FS_FileOpen(FileIndex)==false) return;
    if(StartFPK(FileHandle)==false){ cwl();
      FreeFPK();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    FS_ExecuteFile_InitFileInfoWindow();
    
    FS_Redraw_SetAll();
    MWin_RefreshWindow(WM_FileSelect);
    MWin_TransWindow(WM_FileSelect);
    
    MWin_AllRefresh();
    return;
  }
  
//--------------------------------------------------------
  
  if(ExecMode==EM_MSPSound){ cwl();
    {
      char fn[PluginFilenameMax];
      char ext[ExtMaxLength];
      
      FileSys_GetFileExt(FileIndex,ext);
      DLLList_GetPluginFilename(ext,fn);
      pPluginBody=DLLList_LoadPlugin(fn);
    }
    MWin_AllocMem(WM_PlayControl);
    if(FS_FileOpen(FileIndex)==false) return;
    if(pPluginBody->pSL->Start(FileHandle)==false){ cwl();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    FS_ExecuteFile_InitFileInfoWindow();
    FS_ExecuteFile_InitPlayControlWindow(FileIndex);
    
    u32 SampleRate=pPluginBody->pSL->GetSampleRate();
    u32 SamplePerFrame=pPluginBody->pSL->GetSamplePerFrame();
    u32 ChannelCount=pPluginBody->pSL->GetChannelCount();
    EstrpcmFormat SPF=FS_ExecuteFile_GetOversamplingFactorFromSampleRate(SampleRate);
    
    for(u32 idx=0;idx<SoundEffectPluginCount;idx++){
      pSoundEffectPluginBody[idx]=DLLList_LoadSoundEffectPlugin(idx);
      if(pSoundEffectPluginBody[idx]!=NULL){
        if(pSoundEffectPluginBody[idx]->pSE->Start(SampleRate,SamplePerFrame,ChannelCount)==false){
          DLLList_FreeSoundEffectPlugin(idx);
          safefree(pSoundEffectPluginBody[idx]); pSoundEffectPluginBody[idx]=NULL;
        }
      }
    }
    
    Resume_Backup(true);
    strpcmStart(false,SampleRate,SamplePerFrame,ChannelCount,SPF);
    PrintFreeMem();
    return;
  }
  
#ifdef USEGME

  if(ExecMode==EM_GME){ cwl();
    MWin_AllocMem(WM_PlayControl);
    if(FS_FileOpen(FileIndex)==false) return;
    
    int TrackNum=FileSys_GetFileTrackNum(FileIndex);
    if(TrackNum==-1) TrackNum=0;
    
    char ext[ExtMaxLength];
    FileSys_GetFileExt(FileIndex,ext);
    
    if(StartGME(ext,FileHandle,TrackNum)==false){ cwl();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    FS_ExecuteFile_InitFileInfoWindow();
    FS_ExecuteFile_InitPlayControlWindow(FileIndex);
    
    u32 SampleRate=GME_GetSampleRate();
    u32 SamplePerFrame=GME_GetSamplePerFrame();
    u32 ChannelCount=GME_GetChannelCount();
    EstrpcmFormat SPF=FS_ExecuteFile_GetOversamplingFactorFromSampleRate(SampleRate);
    
    for(u32 idx=0;idx<SoundEffectPluginCount;idx++){
      pSoundEffectPluginBody[idx]=DLLList_LoadSoundEffectPlugin(idx);
      if(pSoundEffectPluginBody[idx]!=NULL){
        if(pSoundEffectPluginBody[idx]->pSE->Start(SampleRate,SamplePerFrame,ChannelCount)==false){
          DLLList_FreeSoundEffectPlugin(idx);
          safefree(pSoundEffectPluginBody[idx]); pSoundEffectPluginBody[idx]=NULL;
        }
      }
    }
    
    Resume_Backup(true);
    strpcmStart(false,SampleRate,SamplePerFrame,ChannelCount,SPF);
    PrintFreeMem();
    return;
  }
  
#endif

//--------------------------------------------------------
  
#ifdef USEDPG
  
  if(ExecMode==EM_DPG){ cwl();
    MWin_AllocMem(WM_PlayControl);
    MWin_AllocMem(WM_PicView);
    
    if(FS_FileOpen2(FileIndex)==false) return;
    if(StartDPG(FileHandle1,FileHandle2)==false){ cwl();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    DPG_RequestSyncStart=true;
    
    FS_ExecuteFile_SetWindowTitle(WM_PicView,FileIndex,"MovieView...");
    
    { cwl();
      MWin_SetVideoMode(ESMM_ForARM7);
      MWin_ClearVideoOverlay();
      MWin_SetVideoOverlaySize(DPG_GetWidth(),DPG_GetHeight(),0);
      MWin_SetVideoOverlay(true);
      MWin_RefreshVideoFullScreen(false);
    }
    
    { cwl();
      MWin_SetSBarH(WM_PicView,ScreenWidth/2,ScreenWidth/2,0);
      MWin_SetSBarV(WM_PicView,ScreenHeight/2,ScreenHeight/2,0);
      
      MWin_SetVisible(WM_PicView,true);
      MWin_SetVisible(WM_PlayControl,true);
      
      MWin_SetActive(WM_PicView);
      MWin_SetActive(WM_PlayControl);
      
      MWin_RefreshScreenMask();
      MWin_AllRefresh();
    }
    
    FS_ExecuteFile_InitFileInfoWindow();
    Resume_Backup(true);
    return;
  }
  
#endif
  
  if(ExecMode==EM_Text){ cwl();
    MWin_AllocMem(WM_TextView);
    if(FS_FileOpen(FileIndex)==false) return;
    if(StartText(FileHandle,MWin_GetClientWidth(WM_TextView),MWin_GetClientHeight(WM_TextView))==false){ cwl();
      FreeText();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    FS_ExecuteFile_InitFileInfoWindow();
    MWin_SetSBarV(WM_TextView,Text_GetLineCount()+1,Text_GetDispLineCount(),Text_GetLine());
    
    MWin_SetVisible(WM_TextView,true);
    u32 WinIndex=MWin_SetActive(WM_TextView);
    MWin_RefreshScreenMask();
    if(WinIndex!=(u32)-1) MWin_RefreshWindow(WinIndex);
    MWin_RefreshWindow(WM_TextView);
    
    switch(GlobalINI.TextPlugin.SelectDisplay){
      case EITPSD_Bottom: IPC3->LCDPowerControl=LCDPC_ON_BOTTOM; break;
      case EITPSD_Top: IPC3->LCDPowerControl=LCDPC_ON_TOP_LEDON; break;
      case EITPSD_DualScreen: IPC3->LCDPowerControl=LCDPC_ON_BOTH; break;
      default: break;
    }
    
    Bookmark_Start();
    Bookmark_CurrentResumeRestore();
    Resume_Backup(true);
    
    if(MSEList_isExists(MSEFN_Text)==true){
      if(Text_GetLine()==0) Text_DrawPluginMessage();
    }
    
    return;
  }
  
  if(ExecMode==EM_BinView){ cwl();
    MWin_AllocMem(WM_BinView);
    if(FS_FileOpen(FileIndex)==false) return;
    if(StartBinView(FileHandle,MWin_GetClientWidth(WM_BinView),MWin_GetClientHeight(WM_BinView))==false){ cwl();
      FreeBinView();
      FS_ExecuteFile_LoadFail();
      PlugInfo_ShowCustomMessage_LoadError();
      FS_ExecuteFile_InitFileInfoWindow();
      return;
    }
    FS_ExecuteFile_InitFileInfoWindow();
    MWin_SetSBarV(WM_BinView,BinView_GetLineCount()+1,BinView_GetDispLineCount(),BinView_GetLine());
    
    MWin_SetVisible(WM_BinView,true);
    u32 WinIndex=MWin_SetActive(WM_BinView);
    MWin_RefreshScreenMask();
    if(WinIndex!=(u32)-1) MWin_RefreshWindow(WinIndex);
    MWin_RefreshWindow(WM_BinView);
    
    switch(GlobalINI.TextPlugin.SelectDisplay){
      case EITPSD_Bottom: IPC3->LCDPowerControl=LCDPC_ON_BOTTOM; break;
      case EITPSD_Top: IPC3->LCDPowerControl=LCDPC_ON_TOP_LEDON; break;
      case EITPSD_DualScreen: IPC3->LCDPowerControl=LCDPC_ON_BOTH; break;
      default: break;
    }
    
    return;
  }
  
//--------------------------------------------------------
  
  if(ExecMode!=EM_None){ cwl();
    MWin_AllocMem(WM_PicView);
    
    switch(ExecMode){ cwl();
      case EM_MSPImage:
        {
          char fn[PluginFilenameMax];
          char ext[ExtMaxLength];
          
          FileSys_GetFileExt(FileIndex,ext);
          DLLList_GetPluginFilename(ext,fn);
          pPluginBody=DLLList_LoadPlugin(fn);
        }
        if(FS_FileOpen(FileIndex)==false) return;
        if(pPluginBody->pIL->Start(FileHandle)==false){ cwl();
          pPluginBody->pIL->Free();
          FS_ExecuteFile_LoadFail();
          PlugInfo_ShowCustomMessage_LoadError();
          FS_ExecuteFile_InitFileInfoWindow();
          return;
        }
        break;
      default: {
        _consolePrint("no-regist ExecMode type.\n");
        ShowLogHalt();
      } break;
    }
    
    PrintFreeMem();
    
    ImageControlTimeOut_SetDefault(GlobalINI.ImagePlugin.GUITimeOutSec);
    imgcalc_SetImageMode(GlobalINI.ImagePlugin.ImageMode,GlobalINI.ImagePlugin.VerticalPadding);
    
    Pic_InitPos();
    
    const float startratio=1;
    
    Pic_SetRatio(startratio);
    
    { cwl();
      if(imgcalc_GetWideFlag()==false){
        MWin_SetVideoMode(ESMM_Normal);
        }else{
        MWin_SetVideoMode(ESMM_Wide);
      }
      MWin_ClearVideoOverlay();
      MWin_SetVideoOverlaySize(0,0,Pic_GetRatio());
      MWin_SetVideoOverlay(true);
      MWin_RefreshVideoFullScreen(true);
    }
    pScreenMain->Flip(true);
    
    Pic_SetSBar(startratio);
    
    FS_ExecuteFile_SetWindowTitle(WM_PicView,FileIndex,"PictureView...");
    
    MWin_SetVisible(WM_PicView,true);
    u32 WinIndex=MWin_SetActive(WM_PicView);
    MWin_RefreshScreenMask();
    if(WinIndex!=(u32)-1) MWin_RefreshWindow(WinIndex);
    FS_ExecuteFile_PicView_DrawFileInfo();
    MWin_RefreshWindow(WM_PicView);
    
    Resume_Backup(true);
    return;
  }
  
}

void FS_StartFromIndex(s32 idx,EExecMode OverrideExecMode)
{ cwl();
  if((0<=idx)&&(idx<FS_IndexCount)){ cwl();
    FS_Redraw_SetPlayIndex();
    FS_PlayIndex=idx;
    FS_Redraw_SetPlayIndex();
    
    MWin_DrawClient(WM_FileSelect);
    if(PluginBodyClock_Execute==false) MWin_TransWindow(WM_FileSelect);
    
    u32 FileType=FileSys_GetFileType(idx);
    
    if(FileType==FT_Path){
      if(OverrideExecMode==EM_None) FS_ExecutePath(idx);
    }
    if(FileType==FT_File) FS_ExecuteFile(idx,OverrideExecMode);
  }
}

static void FS_Restart(void)
{ cwl();
  if(FS_PlayIndex==-1) return;
  
  FS_StartFromIndex(FS_PlayIndex,EM_None);
}

static void FS_StartFromCursor(void)
{ cwl();
  FS_StartFromIndex(FS_CursorIndex,EM_None);
}

static void FS_SetPlayIndex(s32 idx)
{ cwl();
  if(FS_IndexCount<0) idx=0;
  if(FS_IndexCount<=idx) idx=FS_IndexCount-1;
  
  FS_Redraw_SetPlayIndex();
  FS_PlayIndex=idx;
  FS_Redraw_SetPlayIndex();
}

static void FS_MovePlayIndex(s32 v)
{ cwl();
  if(FS_PlayIndex==-1) return;
  
  FS_SetPlayIndex(FS_PlayIndex+v);
}

void FS_SetCursor(s32 idx)
{ cwl();
  if(idx<0) idx=0;
  if((s32)FS_IndexCount<=idx) idx=FS_IndexCount-1;
  
  FS_Redraw_SetCursorIndex();
  FS_CursorIndex=idx;
  FS_Redraw_SetCursorIndex();
  
  if(GlobalINI.System.FileSelectSubScreen==true) thumbChangeCursorIndex(idx);
  
  s32 curidx=FS_CursorIndex;
  s32 lasttop=FS_TopIndex;
  s32 newtop=lasttop;
  s32 dispcnt=FS_DispCount;
  
  if(thumbExists()==false){
    if(dispcnt<=(curidx-newtop)){ cwl();
      newtop=curidx-(dispcnt-1);
      if(newtop<0) newtop=0;
    }
    if((curidx-newtop)<0){ cwl();
      newtop=curidx;
    }
    }else{
    
    if(dispcnt<=(curidx-newtop)){ cwl();
      newtop=curidx;
      newtop=newtop/3*3;
      newtop-=dispcnt-3;
      if(newtop<0) newtop=0;
    }
    if((curidx-newtop)<0){ cwl();
      newtop=curidx;
      newtop=newtop/3*3;
    }
  }
  
  if(lasttop!=newtop){
    FS_TopIndex=newtop;
    FS_Redraw_SetTopIndex();
  }
  
}

static void FS_MoveCursor(s32 v)
{ cwl();
  FS_SetCursor(FS_CursorIndex+v);
}

static void FS_MusicNext(EiniSystemMusicNext MusicNext)
{ cwl();
  if(FS_PlayIndex==-1) return;
  
  FS_Redraw_SetPlayIndex();
  
  EExecMode TagEM;
  
  TagEM=ExecMode;
  if(TagEM==EM_None) TagEM=LastExecMode;
  
  if(TagEM==EM_None) return;
  if(TagEM==EM_MP3Boot) return;
  if(TagEM==EM_BinView) return;
  
  switch(MusicNext){ cwl();
    case EISMN_Stop: { cwl();
    } break;
    case EISMN_Repeat: { cwl();
      FS_StartFromIndex(FS_PlayIndex,EM_None);
    } break;
    case EISMN_NormalLoop: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Normal_GetNextIndex(LastIndex);
        if(FS_PlayIndex==-1) FS_PlayIndex=0;
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_NormalPOff: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Normal_GetNextIndex(LastIndex);
        if(FS_PlayIndex==-1){
          IPC3->LCDPowerControl=LCDPC_SOFT_POWEROFF;
          while(1) swiWaitForVBlank();
        }
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_ShuffleLoop: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Shuffle_GetNextIndex(LastIndex);
        if(FS_PlayIndex==-1){
          Shuffle_Clear();
          FS_PlayIndex=Shuffle_GetNextIndex(LastIndex);
        }
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_ShufflePOff: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Shuffle_GetNextIndex(LastIndex);
        if(FS_PlayIndex==-1){
          IPC3->LCDPowerControl=LCDPC_SOFT_POWEROFF;
          while(1) swiWaitForVBlank();
        }
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_PowerOff: { cwl();
      IPC3->LCDPowerControl=LCDPC_SOFT_POWEROFF;
      while(1) swiWaitForVBlank();
    } break;
    case EISMN_Count: break;
  }
}

static void FS_MusicPrev(EiniSystemMusicNext MusicNext)
{ cwl();
  if(FS_PlayIndex==-1) return;
  
  FS_Redraw_SetPlayIndex();
  
  EExecMode TagEM;
  
  TagEM=ExecMode;
  if(TagEM==EM_None) TagEM=LastExecMode;
  
  if(TagEM==EM_None) return;
  if(TagEM==EM_MP3Boot) return;
  if(TagEM==EM_BinView) return;
  
  switch(MusicNext){ cwl();
    case EISMN_Stop: { cwl();
      _consolePrint("FatalError.MusicPrev_Stop\n");
      ShowLogHalt();
    } break;
    case EISMN_Repeat: { cwl();
      FS_StartFromIndex(FS_PlayIndex,EM_None);
    } break;
    case EISMN_NormalLoop: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Normal_GetPrevIndex(LastIndex);
        if(FS_PlayIndex==-1) break;
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_NormalPOff: { cwl();
      _consolePrint("FatalError.MusicPrev_NormalPOff\n");
      ShowLogHalt();
    } break;
    case EISMN_ShuffleLoop: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Shuffle_GetPrevIndex(LastIndex);
        if(FS_PlayIndex==-1) break;
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_ShufflePOff: { cwl();
      while(1){ cwl();
        int LastIndex=FS_PlayIndex;
        
        FS_PlayIndex=Shuffle_GetNextIndex(LastIndex);
        if(FS_PlayIndex==-1){
          IPC3->LCDPowerControl=LCDPC_SOFT_POWEROFF;
          while(1) swiWaitForVBlank();
        }
        
        if(FileSys_GetFileType(FS_PlayIndex)==FT_File){ cwl();
          if(TagEM==FS_GetExecMode(FS_PlayIndex)){ cwl();
            FS_StartFromIndex(FS_PlayIndex,EM_None);
            break;
          }
        }
      }
    } break;
    case EISMN_PowerOff: { cwl();
      _consolePrint("FatalError.MusicPrev_PowerOff\n");
      ShowLogHalt();
    } break;
    case EISMN_Count: break;
  }
}


