
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <NDS.h>
#include <NDS/ARM9/CP15.h>

#include "../../ipc3.h"

#include "../emulator.h"
#include "../_const.h"
#include "../_console.h"
#include "../_consoleWriteLog.h"
#include "../glib/glib.h"
#include "../memtool.h"
#include "../arm9tcm.h"
#include "../inifile.h"
#include "../mwin.h"

#include "clibdpg.h"
#include "_dpgfs.h"

#ifndef USEDPG

Clibmpg::Clibmpg(CStream *_pCStream,const u32 _TotalFrameCount,const u32 _FPS,const u32 _SndFreq,const EMPGPixelFormat _PixelFormat):TotalFrameCount(_TotalFrameCount),FPS(_FPS),SndFreq(_SndFreq),PixelFormat(_PixelFormat)
{
}

Clibmpg::~Clibmpg(void)
{
}

void Clibmpg::Reopen(u32 StartFrame,u32 StartOffset)
{
}

int Clibmpg::ProcReadBuffer(void)
{
  return(0);
}

bool Clibmpg::ProcSequence(void)
{
  return(false);
}

int Clibmpg::GetWidth(void) const
{
  return(0);
}

int Clibmpg::GetHeight(void) const
{
  return(0);
}

bool Clibmpg::ProcMoveFrame(u32 TargetFrame,u64 TargetSamplesCount)
{
  return(false);
}

bool Clibmpg::ProcMoveFrameGOP(u32 TargetFrame,u64 TargetSamplesCount,u32 TargetGOPFrame,u32 TargetGOPOffset)
{
  return(false);
}

void Clibmpg::SetVideoDelayms(int ms)
{
}

bool Clibmpg::ProcDecode(u64 CurrentSamplesCount)
{
  return(false);
}

int Clibmpg::GetFrameNum(void)
{
  return(0);
}

void Clibmpg::debug_tableout(void)
{
}

void Clibmpg::YUV420toBGR15_Init(int Width,int Height)
{
}

void Clibmpg::YUV420toBGR15_CopyFull(u8 *_YBuf,u8 *_UBuf,u8 *_VBuf,u16 *_FrameBuf)
{
}

#define DECLARE_COPY(func,_LU,_RU,_LD,_RD) \
void func(const u8 *_YBuf,const u8 *_UBuf,const u8 *_VBuf,u16 *_FrameBuf) \
{ \
}

DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyFull,1,1,1,1)
DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyLU,1,0,0,0)
DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyRU,0,1,0,0)
DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyLD,0,0,1,0)
DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyRD,0,0,0,1)

#undef DECLARE_COPY

#define DECLARE_COPY(func) \
void func(const u8 *_YBuf,const u8 *_UBuf,const u8 *_VBuf,u16 *_FrameBuf) \
{ \
}

DECLARE_COPY (Clibmpg::YUV420toBGR15Fix_CopyHalf)

#undef DECLARE_COPY

#else

extern "C" {
__declspec(noreturn) void _ShowLogHalt(void);
}

#define ShowLogHalt() { \
  _ShowLogHalt(); \
  while(1); \
}

extern u32 reqflip;

typedef struct {
  // bool AdaptiveDelayControl;
  u32 msGOPSkip;
  u32 msFull,msHalf,msQuater;
} TAjuster;

static TAjuster stAjuster;

#define ReadBufSize (32*1024)

#include "clibmpg_diskcache.h"
#include "clibmpg_framecache.h"

Clibmpg::Clibmpg(CStream *_pCStream,const u32 _TotalFrameCount,const u32 _FPS,const u32 _SndFreq,const EMPGPixelFormat _PixelFormat):TotalFrameCount(_TotalFrameCount),FPS(_FPS),SndFreq(_SndFreq),PixelFormat(_PixelFormat)
{
  Initialized=false;
  
  {
    TAjuster Ajuster;
    
    //Ajuster.AdaptiveDelayControl=GlobalINI.DPGPlugin.AdaptiveDelayControl;
    
    Ajuster.msGOPSkip=0x7fffffff;
    Ajuster.msFull=0x7fffffff;
    Ajuster.msHalf=0x7fffffff;
    Ajuster.msQuater=0x7fffffff;
    
    if(GlobalINI.DPGPlugin.GOPSkip==true) Ajuster.msGOPSkip=400;
    
    //GlobalINI.DPGPlugin.Dithering=EDPGD_HalfCnt;
    switch(GlobalINI.DPGPlugin.Dithering){
      case EDPGD_NoCnt: {
        // Full only
      } break;
      case EDPGD_HalfCnt: {
        Ajuster.msFull=200;
        Ajuster.msHalf=400;
        Ajuster.msQuater=400;
      } break;
      case EDPGD_QuatCnt: {
        Ajuster.msFull=200;
        Ajuster.msHalf=400;
      } break;
      case EDPGD_HeavyCnt: {
        Ajuster.msFull=100;
        Ajuster.msHalf=200;
        Ajuster.msQuater=300;
      } break;
    }
    
    stAjuster=Ajuster;
  }
  
  MemSet32DMA3(RGB15(0,0,0)|BIT(15),pScreenMain->GetVRAMBuf(ScrMainID_Back),ScreenWidth*ScreenHeight*2);
  MemSet32DMA3(RGB15(0,0,0)|BIT(15),pScreenMain->GetVRAMBuf(ScrMainID_View),ScreenWidth*ScreenHeight*2);
  
  DiskCache_Init();
  DiskCache_LoadAllBuffer();
  
  pCStream=_pCStream;
  DataTopPosition=pCStream->GetOffset();
  
  ReadBuf=(u8*)malloc(ReadBufSize);
  
  framenum=0;
  
  decoder=mpeg2_init();
  if(decoder==NULL){
    _consolePrint("Could not allocate a decoder object.\n");
    return;
  }
  info=mpeg2_info(decoder);
  
  if(ProcSequence()==false) return;
  
  FrameCache_Init(Width*Height*2);
  
  Initialized=true;
}

Clibmpg::~Clibmpg(void)
{
  if(Initialized==true){
    Initialized=false;
    
    if(decoder!=NULL){
      mpeg2_close(decoder); decoder=NULL;
    }
    info=NULL;
    
    if(ReadBuf!=NULL){
      free(ReadBuf); ReadBuf=NULL;
    }
    
    FrameCache_Free();
    DiskCache_Free();
  }
}

void Clibmpg::Reopen(u32 StartFrame,u32 StartOffset)
{
  if(Initialized==true){
    Initialized=false;
    
    if(decoder!=NULL){
      mpeg2_close(decoder); decoder=NULL;
    }
    info=NULL;
  }
  
  framenum=StartFrame;
  
  //pCStream->SetOffset(DataTopPosition+(StartOffset&~3));
  DPGFS_Movie_SetOffset(StartOffset&~3);
  DiskCache_LoadAllBuffer();
  
  decoder=mpeg2_init();
  if(decoder==NULL){
    _consolePrint("Could not allocate a decoder object.\n");
    return;
  }
  info=mpeg2_info(decoder);
  
  if(ProcSequence()==false) return;
  
  REG_IME=0;
  FrameCache_Clear();
  REG_IME=1;
  
  Initialized=true;
}

int Clibmpg::ProcReadBuffer(void)
{
  //_consolePrintf("mpeg read buffer %d %d->%d\n",framenum,pCStream->GetOffset(),ReadBufSize);
  
  //PrfStart();
  //int readsize=pCStream->ReadBuffer32bit(ReadBuf,ReadBufSize);
  //int readsize=DPGFS_Movie_Read16bit(ReadBuf,ReadBufSize);
  int readsize=DiskCache_ReadOneBlock(ReadBuf);
  //PrfEnd(framenum);
  if(readsize!=0) mpeg2_buffer (decoder, ReadBuf,&ReadBuf[readsize]);
  
  return(readsize);
}

bool Clibmpg::ProcSequence(void)
{
  while(1){
    mpeg2_state_t state=mpeg2_parse(decoder);

    switch (state) {
      case STATE_BUFFER: {
        if(ProcReadBuffer()==0) return(false);
      } break;
      case STATE_SEQUENCE: {
        Width=info->sequence->width;
        Height=info->sequence->height;
        int ChromaWidth=info->sequence->chroma_width;
        int ChromaHeight=info->sequence->chroma_height;
        _consolePrintf("ProcSeq:Width=%d,Height=%d\n",Width,Height);
        _consolePrintf("ProcSeq:ChromaWidth=%d,Height=%d\n",ChromaWidth,ChromaHeight);
        YUV420toBGR15_Init(Width,Height);
        return(true);
      } break;
      case STATE_GOP: break;
      case STATE_SLICE: {
        _consolePrint("ProcSeq:StateError found first.(Slice/End/InvalidEnd)\n");
        return(false);
      } break;
      default: break;
    }
  }
}

int Clibmpg::GetWidth(void) const
{
  return(Width);
}

int Clibmpg::GetHeight(void) const
{
  return(Height);
}

bool Clibmpg::ProcMoveFrame(u32 TargetFrame,u64 TargetSamplesCount)
{
  mpeg2_skip(decoder,false);
  if(framenum==TargetFrame) return(true);
  
  bool prgbar;
  bool defull;
  
  if(TargetFrame<framenum){
    Reopen(0,0);
    prgbar=true;
    }else{
    if((FPS*5/0x100)<=(TargetFrame-framenum)){
      prgbar=true;
      }else{
      prgbar=false;
    }
  }
  
  if(TargetFrame==0) return(true);
  
  defull=false;
  
  if(prgbar==true){
    if(MWin_GetVideoFullScreen()==true){
      extern void ProcessTouchPad_DPG(void);
      ProcessTouchPad_DPG();
      defull=true;
    }
  }
  
  mpeg2_skip(decoder,true);
  
  int prgofs=framenum;
  
  if(prgbar==true) MWin_ProgressShow("seek frame...",TargetFrame-prgofs);
  
  while(1){
    mpeg2_state_t state=mpeg2_parse(decoder);
    
    switch (state) {
      case STATE_BUFFER: {
        if(ProcReadBuffer()==0) return(false);
      } break;
      case STATE_SEQUENCE: break;
      case STATE_GOP: break;
      case STATE_SLICE: {
        framenum++;
        if((framenum&63)==0){
          if(prgbar==true) MWin_ProgressSetPos(framenum-prgofs);
        }
        if(TargetFrame<=framenum){
          mpeg2_skip(decoder,false);
          if(prgbar==true) MWin_ProgressHide();
          if(defull==true){
            MWin_SetVideoFullScreen(true);
            MWin_RefreshVideoFullScreen(false);
          }
          return(true);
        }
      } break;
      default: break;
    }
  }
  
  return(false);
}

bool Clibmpg::ProcMoveFrameGOP(u32 TargetFrame,u64 TargetSamplesCount,u32 TargetGOPFrame,u32 TargetGOPOffset)
{
  _consolePrintf("framenum=%d TargetFrame=%d\n",framenum,TargetFrame);
  
  if(framenum==TargetFrame) return(true);
  
  if((TargetGOPFrame<=framenum)&&(framenum<=TargetFrame)){
    }else{
    Reopen(TargetGOPFrame,TargetGOPOffset&~3);
  }
  
  if(framenum==TargetFrame) return(true);
  
  bool prgbar;
  bool defull;
  
  if((FPS*2/0x100)<(TargetFrame-framenum)){
    prgbar=true;
    }else{
    prgbar=false;
  }
  
  defull=false;
  
  if(prgbar==true){
    if(MWin_GetVideoFullScreen()==true){
      extern void ProcessTouchPad_DPG(void);
      ProcessTouchPad_DPG();
      defull=true;
    }
  }
  
  int prgofs=framenum;
  
  if(prgbar==true) MWin_ProgressShow("seek frame...",TargetFrame-prgofs);
  
  while(1){
    mpeg2_state_t state=mpeg2_parse(decoder);
    
    switch (state) {
      case STATE_BUFFER: {
        if(ProcReadBuffer()==0) return(false);
      } break;
      case STATE_SEQUENCE: break;
      case STATE_GOP: break;
      case STATE_SLICE: {
        framenum++;
        if((framenum&3)==0){
          if(prgbar==true) MWin_ProgressSetPos(framenum-prgofs);
        }
        if(TargetFrame<=framenum){
          if(prgbar==true) MWin_ProgressHide();
          if(defull==true){
            MWin_SetVideoFullScreen(true);
            MWin_RefreshVideoFullScreen(false);
          }
          return(true);
        }
      } break;
      default: break;
    }
  }
  
  return(false);
}

extern vu64 DPGAudioStream_SyncSamples;

extern void UpdateDPG_Audio(void);

bool Clibmpg::ProcDecode(void)
{
  UpdateDPG_Audio();
  
  {
    u64 framesamples=(u64)framenum*SndFreq*0x100/FPS;
    
    FrameCache_WriteSetShowSamples(framesamples);
    
    while(DPGAudioStream_SyncSamples<framesamples){
      if(DiskCache_LoadOneBuffer()==false) break;
    }
    
    if(DPGAudioStream_SyncSamples<framesamples){
      if(FrameCache_isWriteFull()==true) return(true);
    }
  }
  
  framenum++;
  if(TotalFrameCount<=framenum){
    _consolePrint("End of video stream.\n");
    return(false);
  }
//  _consolePrintf("Delay=%5dms\n",delayms);
  
  static bool skipflag=false;
  
  while(1){
    mpeg2_state_t state=mpeg2_parse(decoder);
    
    UpdateDPG_Audio();
    
    switch (state) {
      case STATE_BUFFER: {
        if(ProcReadBuffer()==0) return(false);
      } break;
      case STATE_SEQUENCE: break;
      case STATE_GOP: {
        u64 framesamples=FrameCache_WriteGetShowSamples();
        if(DPGAudioStream_SyncSamples<framesamples){
          if(skipflag==true){
            skipflag=false;
            mpeg2_skip(decoder,skipflag);
          }
          }else{
          s32 LastDelaySamples=(s32)(DPGAudioStream_SyncSamples-framesamples);
          s32 delayms=LastDelaySamples*1000/SndFreq;
          if(stAjuster.msGOPSkip<delayms){
            if(skipflag==false){
              skipflag=true;
              mpeg2_skip(decoder,skipflag);
              _consolePrint("DropGOP");
            }
          }
        }
      } break;
      case STATE_SLICE: {
        if((skipflag==false)&&(info->display_fbuf)){
          if(DPGAudioStream_SyncSamples<FrameCache_WriteGetShowSamples()){
            // frame cache mode
            u16 *pVRAMBuf=FrameCache_WriteStart();
            YUV420toBGR15Fix_CopyFull(info->display_fbuf->buf[0],info->display_fbuf->buf[1],info->display_fbuf->buf[2],pVRAMBuf);
            FrameCache_WriteEnd();
            }else{
            // direct VRAM mode
            REG_IME=0;
            if(reqflip!=0){
              if(reqflip==3) pScreenMain->Flip(false);
              pScreenMain->SetBlendLevel(16);
              reqflip=0;
            }
            REG_IME=1;
            
            // ARM9 processing converter
            bool reqtrans=true;
            
            u16 *pVRAMBuf=pScreenMain->GetVRAMBuf(ScrMainID_Back);
            
            { // Enable VRAM buffer write cache
              u32 r0=7;
              __asm {
                mcr p15, 0, r0, c3, c0, 0
              }
            }
            
            if(Width!=ScreenWidth){
              YUV420toBGR15_CopyFull(info->display_fbuf->buf[0],info->display_fbuf->buf[1],info->display_fbuf->buf[2],pVRAMBuf);
              }else{
              s32 LastDelaySamples=(s32)(DPGAudioStream_SyncSamples-FrameCache_WriteGetShowSamples());
              s32 delayms=LastDelaySamples*1000/SndFreq;
              //_consolePrintf("delay=%8dms\n",delayms);
              TAjuster Ajuster=stAjuster;
              if(delayms<Ajuster.msFull){
                //PrfStart();
                YUV420toBGR15Fix_CopyFull(info->display_fbuf->buf[0],info->display_fbuf->buf[1],info->display_fbuf->buf[2],pVRAMBuf);
                //PrfEnd(0);
                }else{
                if(delayms<Ajuster.msHalf){
                  YUV420toBGR15Fix_CopyHalf(info->display_fbuf->buf[0],info->display_fbuf->buf[1],info->display_fbuf->buf[2],pVRAMBuf);
                  }else{
                  if(delayms<Ajuster.msQuater){
                    YUV420toBGR15Fix_CopyQuat(info->display_fbuf->buf[0],info->display_fbuf->buf[1],info->display_fbuf->buf[2],pVRAMBuf);
                    }else{
                    reqtrans=false;
                    _consolePrint("!");
                  }
                }
              }
            }
            
            { // Disable VRAM buffer write cache
              u32 r0=6;
              __asm {
                mcr p15, 0, r0, c3, c0, 0
              }
            }
            
            if(reqtrans==true) reqflip=3;
          }
        }
        
        UpdateDPG_Audio();
        return(true);
      } break;
      default: break;
    }
  }
  
  return(false);
}

int Clibmpg::GetFrameNum(void)
{
  return(framenum);
}


// -----------------------------------------------------------------

// all format
//#define trofs (197)
//#define tgofs (132)
//#define tbofs (232)

// SMPTE 170M format optimized.
#define trofs (175/2)
#define tgofs (132/2)
#define tbofs (221/2)

// no limitter
//#define trofs (0)
//#define tgofs (0)
//#define tbofs (0)

typedef u8 TColorR;
typedef u32 TColorG;
typedef u32 TColorB;
typedef u32 TColorRGB;

#include "tcmstart.h"
static TColorR itable_rbuf[256+trofs*2];
static TColorG itable_gbuf[256+tgofs*2];
static TColorB itable_bbuf[256+tbofs*2];
static TColorR *itable_rV[256];
static TColorG *itable_gU[256];
static u8 itable_gV[256];
static TColorB *itable_bU[256];
#include "tcmend.h"

static __forceinline int div_round (int dividend, int divisor)
{
    if (dividend > 0)
        return (dividend + (divisor>>1)) / divisor;
    else
        return -((-dividend + (divisor>>1)) / divisor);
}

void Clibmpg::debug_tableout(void)
{
}

void Clibmpg::YUV420toBGR15_Init(int Width,int Height)
{
  TYUV420toBGR15_DATA *id=&YUV420toBGR15_DATA;
  
  u8 table_Y[1024];
  
  u32 bs;
  switch(PixelFormat){
    case PF_RGB15: bs=0; break;
    case PF_RGB18: bs=1; break;
    case PF_RGB21: bs=2; break;
    case PF_RGB24: bs=3; break;
    default: bs=3; break;
  }
  
  for (int i=0;i<1024;i++){
    int j=((76309*(i-384-16))+32768) >> 16;
    j=(j<0) ? 0 : ((j>255) ? 255 : j);
    j>>=bs;
    if(0x1f<j) j=0x1f;
    table_Y[i]=j;
  }

  for (int i=-trofs;i<256+trofs;i++){
    itable_rbuf[i+trofs]=table_Y[i+384];
  }
  
  for (int i=-tgofs;i<256+tgofs;i++){
    itable_gbuf[i+tgofs]=table_Y[i+384] << 5;
  }
  
  for (int i=-tbofs;i<256+tbofs;i++){
    itable_bbuf[i+tbofs]=(table_Y[i+384] << 10) | (1 << 15); // transmask
  }
  
#if 0
  int matrix_coefficients=6;
  int Inverse_Table_6_9[8][4] = {
    {117504, 138453, 13954, 34903}, /* no sequence_display_extension */
    {117504, 138453, 13954, 34903}, /* ITU-R Rec. 709 (1990) */
    {104597, 132201, 25675, 53279}, /* unspecified */
    {104597, 132201, 25675, 53279}, /* reserved */
    {104448, 132798, 24759, 53109}, /* FCC */
    {104597, 132201, 25675, 53279}, /* ITU-R Rec. 624-4 System B, G */
    {104597, 132201, 25675, 53279}, /* SMPTE 170M */
    {117579, 136230, 16907, 35559}  /* SMPTE 240M (1987) */
  };
#endif
  
  // for SMPTE 170M only
  const int crv=104597; // Inverse_Table_6_9[matrix_coefficients][0];
  const int cbu=132201; // Inverse_Table_6_9[matrix_coefficients][1];
  const int cgu=-25675; // -Inverse_Table_6_9[matrix_coefficients][2];
  const int cgv=-53279; // -Inverse_Table_6_9[matrix_coefficients][3];
  
  for (int i=0;i<256;i++) {
    itable_rV[i]=&itable_rbuf[trofs+div_round(crv*(i-128), 76309)];
    itable_gU[i]=&itable_gbuf[tgofs+div_round(cgu*(i-128), 76309)-89];
    itable_gV[i]=                   div_round(cgv*(i-128), 76309)+89;
    itable_bU[i]=&itable_bbuf[tbofs+div_round(cbu*(i-128), 76309)];
  }
  
//  debug_tableout();
  
  id->width=Width;
  id->height=Height;
  id->Xdiv2=Width/2;
  id->Ydiv2=Height/2;
}

// --------------

#define RGB(i) { \
  const TColorR *r=itable_rV[pv[i]]; \
  const TColorG *g=itable_gU[pu[i]]+itable_gV[pv[i]]; \
  const TColorB *b=itable_bU[pu[i]]; \
  const u32 aofs=ofs+(i*2); \
  { \
    const u32 Y0=py_1[aofs+0],Y1=py_1[aofs+1]; \
    *(u32*)&dst[aofs]=r[Y0] | g[Y0] | b[Y0] | ((r[Y1] | g[Y1] | b[Y1])<<16); \
  } \
  { \
    const u32 Y0=py_2[aofs+0],Y1=py_2[aofs+1]; \
    *(u32*)&dst2[aofs]=r[Y0] | g[Y0] | b[Y0] | ((r[Y1] | g[Y1] | b[Y1])<<16); \
  } \
}

void Clibmpg::YUV420toBGR15_CopyFull(u8 *_YBuf,u8 *_UBuf,u8 *_VBuf,u16 *_FrameBuf)
{
  const TYUV420toBGR15_DATA *id=&YUV420toBGR15_DATA;
  
  u16 *dst=&_FrameBuf[0];
  const u8 *py_1=_YBuf;
  const u8 *pu=_UBuf;
  const u8 *pv=_VBuf;
  u32 dy=id->Ydiv2;
  
  do {
    u16 *dst2=&dst[ScreenWidth];
    const u8 *py_2=&py_1[id->width];
    u32 dx=id->Xdiv2;
    u32 ofs=0;
    do {
      RGB(0);
      ofs+=2;
      pu+=1; pv+=1;
    } while (--dx);
    
    py_1+=id->width*2;
    dst+=ScreenWidth*2;
  } while (--dy);
}

#undef RGB

// --------------------------------------

#define RGB(i) { \
  const TColorR *r=itable_rV[pv[i]]; \
  const TColorG *g=itable_gU[pu[i]]+itable_gV[pv[i]]; \
  const TColorB *b=itable_bU[pu[i]]; \
  const u32 aofs=ofs+(i*2); \
  { \
    const u32 Y0=py[aofs+0],Y1=py[aofs+1]; \
    *(u32*)&dst[aofs]=r[Y0] | g[Y0] | b[Y0] | (((TColorRGB)(r[Y1] | g[Y1] | b[Y1]))<<16); \
  } \
  { \
    const u32 aaofs=aofs+ScreenWidth; \
    const u32 Y0=py[aaofs+0],Y1=py[aaofs+1]; \
    *(u32*)&dst[aaofs]=r[Y0] | g[Y0] | b[Y0] | (((TColorRGB)(r[Y1] | g[Y1] | b[Y1]))<<16); \
  } \
}

#include "tcmstart.h"
#pragma Otime
void Clibmpg::YUV420toBGR15Fix_CopyFull(const u8 *_YBuf,const u8 *_UBuf,const u8 *_VBuf,u16 *_FrameBuf)
{
  const TYUV420toBGR15_DATA *id=&YUV420toBGR15_DATA;
  
  u16 *dst=&_FrameBuf[0];
  const u8 *py=_YBuf;
  const u8 *pu=_UBuf;
  const u8 *pv=_VBuf;
  u32 dy=id->Ydiv2;
  
  do {
    u32 dx=ScreenWidth/4;
    u32 ofs=0;
    do {
      RGB(0); RGB(1);
      ofs+=4;
      pu+=2; pv+=2;
    } while (--dx);
    
    py+=ScreenWidth*2;
    dst+=ScreenWidth*2;
  } while (--dy);
}
#pragma Ospace
#include "tcmend.h"

#undef RGB

// --------------------------------------

//static const u32 DitherTable4[4*4]={0,4,1,5, 6,2,7,3, 1,5,0,4, 7,3,6,2};

// --------------------------------------

#define RGB(i) { \
  const TColorR *r=itable_rV[pv[i]]; \
  const TColorG *g=itable_gU[pu[i]]+itable_gV[pv[i]]; \
  const TColorB *b=itable_bU[pu[i]]; \
  const u32 aofs=ofs+(i*2); \
  { \
    TColorRGB c; \
    { const u32 Y=py[aofs+0]; c=r[Y] | g[Y] | b[Y]; } \
    *(u32*)&dst[aofs]=c | (c<<16); \
  } \
  { \
    const u32 aaofs=aofs+ScreenWidth; \
    TColorRGB c; \
    { const u32 Y=py[aaofs+0]; c=r[Y] | g[Y] | b[Y]; } \
    *(u32*)&dst[aaofs]=c | (c<<16); \
  } \
}

#include "tcmstart.h"
#pragma Otime
void Clibmpg::YUV420toBGR15Fix_CopyHalf(const u8 *_YBuf,const u8 *_UBuf,const u8 *_VBuf,u16 *_FrameBuf)
{
  const TYUV420toBGR15_DATA *id=&YUV420toBGR15_DATA;
  
  u16 *dst=&_FrameBuf[0];
  const u8 *py=_YBuf;
  const u8 *pu=_UBuf;
  const u8 *pv=_VBuf;
  u32 dy=id->Ydiv2;
  
  do {
    u32 dx=ScreenWidth/4;
    u32 ofs=0;
    do {
      RGB(0); RGB(1);
      ofs+=4;
      pu+=2; pv+=2;
    } while (--dx);
    
    py+=ScreenWidth*2;
    dst+=ScreenWidth*2;
  } while (--dy);
}
#pragma Ospace
#include "tcmend.h"

#undef RGB

// --------------------------------------

#define RGB(i) { \
  const TColorR *r=itable_rV[pv[i]]; \
  const TColorG *g=itable_gU[pu[i]]+itable_gV[pv[i]]; \
  const TColorB *b=itable_bU[pu[i]]; \
  const u32 aofs=ofs+(i*2); \
  { \
    TColorRGB c; \
    { const u32 Y=py[aofs+0]; c=r[Y] | g[Y] | b[Y]; } \
    c|=c<<16; \
    *(u32*)&dst[aofs]=c; \
    *(u32*)&dst[aofs+ScreenWidth]=c; \
  } \
}

#include "tcmstart.h"
#pragma Otime
void Clibmpg::YUV420toBGR15Fix_CopyQuat(const u8 *_YBuf,const u8 *_UBuf,const u8 *_VBuf,u16 *_FrameBuf)
{
  const TYUV420toBGR15_DATA *id=&YUV420toBGR15_DATA;
  
  u16 *dst=&_FrameBuf[0];
  const u8 *py=_YBuf;
  const u8 *pu=_UBuf;
  const u8 *pv=_VBuf;
  u32 dy=id->Ydiv2;
  
  do {
    u32 dx=ScreenWidth/4;
    u32 ofs=0;
    do {
      RGB(0); RGB(1);
      ofs+=4;
      pu+=2; pv+=2;
    } while (--dx);
    
    py+=ScreenWidth*2;
    dst+=ScreenWidth*2;
  } while (--dy);
}
#pragma Ospace
#include "tcmend.h"

#undef RGB

// --------------------------------------

#endif

