
//#define FrameCacheDebugOut

#define FrameCacheBodySize (ReadBufSize)
#define FrameCacheBodyCount (8)
#define FrameCacheBodyCountMask ((FrameCacheBodyCount)-1)

typedef struct {
  u64 ShowSamples;
  u16 *pbuf;
} TFrameCacheBody;

typedef struct {
  u32 BufferSizeByte;
  u32 ReadIndex,WriteIndex;
  TFrameCacheBody Body[FrameCacheBodyCount];
} TFrameCache;

static TFrameCache FrameCache;

static void FrameCache_Init(u32 BufferSizeByte)
{
  TFrameCache *pfc=&FrameCache;
  
  pfc->BufferSizeByte=BufferSizeByte;
  pfc->ReadIndex=0;
  pfc->WriteIndex=0;
  
  for(u32 idx=0;idx<FrameCacheBodyCount;idx++){
    TFrameCacheBody *pfcb=&pfc->Body[idx];
    pfcb->ShowSamples=(u64)-1;
    pfcb->pbuf=(u16*)safemalloc(BufferSizeByte);
    if(pfcb->pbuf==NULL){
      _consolePrintf("Memory overflow. can not create FrameCache buffers.\n");
      ShowLogHalt();
    }
  }
}

static void FrameCache_Free(void)
{
  TFrameCache *pfc=&FrameCache;
  
  pfc->ReadIndex=0;
  pfc->WriteIndex=0;
  
  for(u32 idx=0;idx<FrameCacheBodyCount;idx++){
    TFrameCacheBody *pfcb=&pfc->Body[idx];
    pfcb->ShowSamples=0;
    if(pfcb->pbuf!=NULL){
      safefree(pfcb->pbuf); pfcb->pbuf=NULL;
    }
  }
}

static void FrameCache_Clear(void)
{
  TFrameCache *pfc=&FrameCache;
  
  pfc->ReadIndex=0;
  pfc->WriteIndex=0;
}

static bool FrameCache_isReadEmpty(void)
{
  TFrameCache *pfc=&FrameCache;
  
  if(pfc->ReadIndex==pfc->WriteIndex){
#ifdef FrameCacheDebugOut
//    _consolePrint("FrameCache_isReadEmpty=True.\n");
#endif
    return(true);
  }
  
  return(false);
}

static bool FrameCache_isWriteFull(void)
{
  TFrameCache *pfc=&FrameCache;
  
  u32 widx=(pfc->WriteIndex+1)&FrameCacheBodyCountMask;
  if(widx==pfc->ReadIndex){
#ifdef FrameCacheDebugOut
    _consolePrint("FrameCache_isWriteFull=True.\n");
#endif
    return(true);
  }
  
  return(false);
}

static u64 FrameCache_WriteShowSamplesTemp;

static void FrameCache_WriteSetShowSamples(u64 ShowSamples)
{
  FrameCache_WriteShowSamplesTemp=ShowSamples;
}

static u64 FrameCache_WriteGetShowSamples(void)
{
  return(FrameCache_WriteShowSamplesTemp);
}

static u16* FrameCache_WriteStart(void)
{
  TFrameCache *pfc=&FrameCache;
  
  TFrameCacheBody *pfcb=&pfc->Body[pfc->WriteIndex];
  
  pfcb->ShowSamples=FrameCache_WriteShowSamplesTemp;
  return(pfcb->pbuf);
}

static void FrameCache_WriteEnd(void)
{
  TFrameCache *pfc=&FrameCache;
  
  pfc->WriteIndex=(pfc->WriteIndex+1)&FrameCacheBodyCountMask;
}

u32 FrameCache_GetBufferSizeByte(void)
{
  TFrameCache *pfc=&FrameCache;
  
  return(pfc->BufferSizeByte);
}

u16* FrameCache_ReadStart(u64 CurrentSamples)
{
  if(FrameCache_isReadEmpty()==true) return(NULL);
  
  TFrameCache *pfc=&FrameCache;
  
  TFrameCacheBody *pfcb=&pfc->Body[pfc->ReadIndex];
  
  if(CurrentSamples<pfcb->ShowSamples) return(NULL);
  
  return(pfcb->pbuf);
}

void FrameCache_ReadEnd(void)
{
  TFrameCache *pfc=&FrameCache;
  
  pfc->ReadIndex=(pfc->ReadIndex+1)&FrameCacheBodyCountMask;
}

