
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <NDS.h>

#include "../filesys.h"
#include "../memtool.h"

#include "../_console.h"
#include "../_consoleWriteLog.h"

#include "../../../ipc3.h"

#include "plug_mp2.h"
#include "_dpgfs.h"

#define ReadBufSize (8*1024)

static bool Initialized=false;

void FreeMP2(void);

static u64 MoveSample;

bool StartMP2(void)
{
  if(Initialized==true) FreeMP2();
  Initialized=true;
  
  _consolePrint("Request ARM7 mp2 decode.\n");
  
  IPC3->IR=IR_NULL;
  IPC3->IR_filesize=DPGFS_Audio_GetSize();
  IPC3->IR_readsize=0;
  IPC3->IR_readbuf=(u8*)malloc(ReadBufSize);
  IPC3->IR_readbufsize=0;
  IPC3->IR_flash=false;
  IPC3->IR_EOF=false;
  
  IPC3->IR_SyncSamples_SendToARM9=0;
  
  MoveSample=0;
  
  return(true);
}

void FreeMP2(void)
{
  if(Initialized==false) return;
  Initialized=false;
  
  IPC3->IR_filesize=0;
  IPC3->IR_readsize=0;
  if(IPC3->IR_readbuf!=NULL){
    free(IPC3->IR_readbuf); IPC3->IR_readbuf=NULL;
  }
}

void MP2_SetPosition(double per,u64 smp)
{
  if(Initialized==false) return;
  
  if(per<0) per=0;
  MoveSample=smp;
  
  REG_IE&=~IRQ_IPC_SYNC;
  
  u32 MovePosition=(u32)(DPGFS_Audio_GetSize()*per);
  MovePosition&=~1;
  
  u8 *buf=(u8*)IPC3->IR_readbuf;
  
  DPGFS_Audio_SetOffset(MovePosition);
  MovePosition=0;
  
  u32 ReadSize=DPGFS_Audio_Read16bit(&buf[0],ReadBufSize);
  
  if(DPGFS_Audio_GetSize()<=DPGFS_Audio_GetOffset()) IPC3->IR_EOF=true;
  
  IPC3->IR_readbufsize=ReadSize;
  
  IPC3->IR_flash=true;
  
  REG_IE|=IRQ_IPC_SYNC;
}

static inline void IPCSYNC_Disabled(void)
{
  REG_IE&=~IRQ_IPC_SYNC;
}

static inline void IPCSYNC_Enabled(void)
{
  REG_IE|=IRQ_IPC_SYNC;
}

#include "tcmstart.h"
void MP2_LoadReadBuffer(void)
{
  if(Initialized==false) return;
  
  if(IPC3->IR_flash==true) return;

  IPCSYNC_Disabled();
  
  u8 *buf=(u8*)IPC3->IR_readbuf;
  int ReadSize=ReadBufSize-IPC3->IR_readbufsize;
  
  ReadSize&=~1;
  if(ReadSize==0){
    IPCSYNC_Enabled();
    return;
  }
  
  ReadSize=DPGFS_Audio_Read16bit(&buf[IPC3->IR_readbufsize],ReadSize);
  
  if(DPGFS_Audio_GetSize()<=DPGFS_Audio_GetOffset()) IPC3->IR_EOF=true;
  
  IPC3->IR_readbufsize+=ReadSize;
  
  IPCSYNC_Enabled();
}
#include "tcmend.h"

void IRQSYNC_MP2_flash(void)
{
  if(Initialized==false) return;
  IPC3->IR_SyncSamples_SendToARM9=MoveSample;
  MoveSample=0;
}

#include "tcmstart.h"
void IRQSYNC_MP2_fread(void)
{
  //_consolePrintf("fr%d,%d\n",IPC3->IR_readsize,IPC3->IR_readbufsize);
  
  u8 *buf=(u8*)IPC3->IR_readbuf;
  int size=IPC3->IR_readsize;
  int remain=IPC3->IR_readbufsize-size;
  
  if(remain<=0){
    remain=0;
    }else{
    //PrfStart();
    DC_FlushRangeOverrun((void*)buf,ReadBufSize);
    DMA0_SRC = (uint32)&buf[size];
    DMA0_DEST = (uint32)&buf[0];
    DMA0_CR = DMA_ENABLE | DMA_SRC_INC | DMA_DST_INC | DMA_16_BIT | (remain/2);
    //PrfEnd(remain);
//    while(DMA0_CR & DMA_BUSY);
  }
  
  IPC3->IR_readbufsize=remain;
}
#include "tcmend.h"


