
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <NDS.h>

#include "_const.h"
#include "_console.h"

#include "memtool.h"

#include "setting.h"

#include "directdisk.h"

#include "inifile.h"

#include "main_extforresume.h"

static bool SettingEnabled;

#define SettingData_Version (17)

typedef struct {
  u8 INI_MD5Data[16];
  u32 Version;
  TiniCustom Custom;
  TiniSystem System;
  TiniHiddenItem HiddenItem;
  TiniKeyRepeat KeyRepeat;
  TiniBacklightTimeout BacklightTimeout;
  TiniBoot Boot;
  TiniThumbnail Thumbnail;
  TiniTextPlugin TextPlugin;
  TiniImagePlugin ImagePlugin;
  TiniGMEPlugin GMEPlugin;
  char LastPath[SettingData_LastPathLen];
  u8 dummy[DD_SectorSize];
} TSettingData;

static TSettingData SettingData;

void Setting_Init(void)
{
  SettingEnabled=false;
  
  if(DD_SectorSize<(sizeof(TSettingData)-DD_SectorSize)){
    _consolePrintf("overflow in SectorSize<%d sizeof(TSettingData)-DD_SectorSize\n",sizeof(TSettingData)-DD_SectorSize);
    ShowLogHalt();
  }
  
  if(DD_isEnabled()==false) return;
  if(DD_isExists(EDDFile_setting)==false) return;
  
  _consolePrint("\nSetting initialize.\n");
  
  if(DD_InitFile(EDDFile_setting)==false){
    _consolePrint("\nCritical error!! CPU HALT for safety.\n");
    ShowLogHalt();
    while(1);
  }
  
  _consolePrint("Setting initialized.\n");
  
  SettingEnabled=true;
}

void Setting_Backup(void)
{
  if(SettingEnabled==false) return;
  
  TSettingData *psd=&SettingData;
  TGlobalINI *pgi=&GlobalINI;
  
  MemCopy8CPU(INI_MD5Data,psd->INI_MD5Data,16);
  psd->Version=SettingData_Version;
  
  psd->Custom=pgi->Custom;
  psd->System=pgi->System;
  psd->HiddenItem=pgi->HiddenItem;
  psd->KeyRepeat=pgi->KeyRepeat;
  psd->BacklightTimeout=pgi->BacklightTimeout;
  psd->Boot=pgi->Boot;
  psd->Thumbnail=pgi->Thumbnail;
  psd->TextPlugin=pgi->TextPlugin;
  psd->ImagePlugin=pgi->ImagePlugin;
  psd->GMEPlugin=pgi->GMEPlugin;
  
  MemSet32CPU(0xffffffff,psd->dummy,DD_SectorSize);
  
  DD_WriteFile(EDDFile_setting,&SettingData,DD_SectorSize);
}

void Setting_Restore(void)
{
  if(SettingEnabled==false) return;
  
  DD_ReadFile(EDDFile_setting,&SettingData,DD_SectorSize);
  
  if(memcmp(SettingData.INI_MD5Data,INI_MD5Data,16)!=0){
    _consolePrint("detect moonshl.ini changed.\nignore moonshl.sav.\n");
    return;
  }
  
  if(SettingData.Version!=SettingData_Version){
    _consolePrint("detect old version data.\nignore moonshl.sav.\n");
    return;
  }
  
  TSettingData *psd=&SettingData;
  TGlobalINI *pgi=&GlobalINI;
  
  pgi->Custom=psd->Custom;
  pgi->System=psd->System;
  pgi->HiddenItem=psd->HiddenItem;
  pgi->KeyRepeat=psd->KeyRepeat;
  pgi->BacklightTimeout=psd->BacklightTimeout;
  pgi->Boot=psd->Boot;
  pgi->Thumbnail=psd->Thumbnail;
  pgi->TextPlugin=psd->TextPlugin;
  pgi->ImagePlugin=psd->ImagePlugin;
  pgi->GMEPlugin=psd->GMEPlugin;
}

void Setting_Backup_LastPath(const char *pPath)
{
  if(SettingEnabled==false) return;
  if(pPath==NULL) return;
  
  char *pLastPath=SettingData.LastPath;
  
  MemSet8CPU(0,pLastPath,SettingData_LastPathLen);
  strncpy(pLastPath,pPath,SettingData_LastPathLen-1);
  
  DD_WriteFile(EDDFile_setting,&SettingData,DD_SectorSize);
}

const char* Setting_Restore_LastPath(void)
{
  return(SettingData.LastPath);
}

